set ::debug 1

namespace eval wakeRes {
    variable propPos [list 0.0 0.0 0.0]
    variable listJ ""
    variable propDiam 0.0
    variable currStep 0
    variable flowVel 0.0
    variable pi 3.141592653589
    variable incrX 0.1
    variable incrY 0.2
    variable units
    variable progress
    variable countE
	
}
proc wakeRes::UseCreateWakeWindow { {w ""} } {
    global ProblemTypePriv
    global PROJECTPATH CoordsCirc
    variable propDiam
    variable propPos
    variable flowVel
    variable units
    variable incrX
    variable incrY
	
    package require tile

    set units [GiD_AccessValue get gendata Geometry_Units]
    if {$units == "\[m\]"} {
	set incrX 0.1
	set incrY 0.2
    } elseif {$units == "\[mm\]"} {
	set incrX 100.0
	set incrY 200.0
    } elseif {$units == "\[cm\]"} {
	set incrX 10.0
	set incrY 20.0
    } 

    set wresw .gid.wakeresults   
    if { [winfo exists $wresw] } { return }
    InitWindow $wresw [_ "Wake results viewer"] wakeResWindowGeom WakeResView "" 1
    if { ![winfo exists $wresw] } return
    
    set pare [winfo parent $wresw]
    set posx [expr [winfo x $pare]+[winfo width $pare ]/4]
    set posy [expr [winfo y $pare]+[winfo height $pare ]/4]
    wm geometry $wresw [join "+$posx + $posy" ""]
    
    set b $wresw.buts 
    frame $b
    
    ttk::label $b.posL -text "Propeller Position"
    ttk::button $b.pickB -text [= {Pick}] -width 10 -command [list wakeRes::pick] -und 0
    ttk::label $b.posU -text $wakeRes::units

    set posX [entry $b.posX -textvar wakeRes::propPos]

    ttk::label $b.diamL -text "Propeller Diameter"
    set diamE [ttk::entry $b.diamE -textvar wakeRes::propDiam]
    ttk::label $b.diamU -text $wakeRes::units

    ttk::label $b.velL -text "Inflow Velocity"
    set velE [ttk::entry $b.velE -textvar wakeRes::flowVel]
       
    ttk::button $b.pictB -text [= {Include image}] -width 10 -command [list wakeRes::picture] -und 0
    ttk::button $b.apply -text [= {Apply}] -width 5 -und 0 -command [list wakeRes::process]
    ttk::button $b.close -text [= {Close}] -width 5 -und 0 -command [list wakeRes::EndDrawPropeller $wresw]

    wm protocol $wresw WM_DELETE_WINDOW "tkButtonInvoke $b.close"
    bind $wresw <Escape> "tkButtonInvoke $b.close"
    bind $wresw <Alt-c> "tkButtonInvoke $b.close"
    
    grid $b.posL -row 1 -column 0 -sticky news -pady 3 -padx 3
    grid $b.posX -row 1 -column 1 -sticky news -pady 3 -padx 3
    grid $b.posU -row 1 -column 2 -sticky news -pady 3 -padx 3
    grid $b.pickB -row 1 -column 3 -sticky news -pady 3 -padx 3

    grid $b.diamL -row 2 -column 0 -sticky news -pady 3 -padx 3
    grid $b.diamE -row 2 -column 1 -sticky news -pady 3 -padx 3
    grid $b.diamU -row 2 -column 2 -sticky news -pady 3 -padx 3

    grid $b.velL -row 3 -column 0 -sticky news -pady 3 -padx 3
    grid $b.velE -row 3 -column 1 -sticky news -pady 3 -padx 3

    grid $b.apply -row 4 -column 0 -sticky news -pady 3 -padx 3
    grid $b.close -row 4 -column 1 -sticky news -pady 3 -padx 3
    grid $b -padx 3 -pady 3
    grid rowconfigure $wresw {1 2 3} -weight 1
    grid columnconfigure $wresw 0 -weight 1 
    wm geom $wresw ""
}

proc wakeRes::pick {} {
    variable propPos
    variable listVx ""
    
    set aux [GidUtils::GetCoordinates [= "Enter point coordinates"] NoJoin]
    set propPos [format "%.3g %.3g %.3g" [lindex $aux 0] [lindex $aux 1] [lindex $aux 2] ]
}

proc wakeRes::process {} {
    variable propPos
    global ProblemTypePriv 
    global GidPriv
    global PROJECTPATH CoordsCirc
    variable progress
    variable currStep
    variable propDiam
    
    set progress 0
    
    if {$::debug != 1} {
	set wakeBar .gid.wakeprogress
	if { [winfo exists $wakeBar] } { return }
	ProgressDlg::create .gid.wakeprogress -height 2 -width 50 -stop "Stop" -title "Wake Calculation Progress" \
	    -variable wakeRes::progress
	if { ![winfo exists $wakeBar] } return  
	
	set pBar [ProgressBar $wakeBar.pBar -height 2 -maximum 100 -type incremental \
		-variable wakeRes::progress -width 40  -troughcolor grey -bg white -relief raised -orient horizontal]
    }
    
    GidUtils::DisableWarnLine
	
    set analysisAll [GiD_Info postprocess get all_analysis]
    if {$analysisAll == ""} {
	WarnWin "No analysis available"
	return
    }

    set cutsAll [GiD_Info postprocess get all_cuts]
    if {$cutsAll != ""} {
	GiD_Process Mescape Utilities Delete CutSets ALL_CUTSETS
    }

    set surfs [GiD_Info postprocess get all_surfaces]
    set vols [GiD_Info postprocess get all_volumes]

    incr progress
    foreach surf $surfs {
	catch { GiD_Process Mescape Utilities ChangeProperty SurfaceSets $surf Massive No }
    }
    incr progress
    foreach vol $vols {
	catch { GiD_Process Mescape Utilities ChangeProperty VolumeSets $vol Massive No }
    }
    incr progress
    catch { GiD_Process Mescape DisplayStyle BodyBound }
    GiD_Process Mescape View Redraw
	   
    GetVel 0
    CreateResW
	
    set ref 1.0
    set cutPoints ""
    set p1 ""
    set p2 ""
    set p3 ""

    set p1 $propPos
    lappend p2 [lindex $propPos 0]
    lappend p2 [expr [lindex $propPos 1] + $ref]
    lappend p2 [lindex $propPos 2] 

    lappend p3 [lindex $propPos 0] 
    lappend p3 [lindex $propPos 1]
    lappend p3 [expr [lindex $propPos 2] + $ref]
    catch { GiD_Process Mescape DoCut CutPlane ThreePoints $p1 $p2 $p3 }

    CalcNomWake
  
    CreatePropeller  
    CreateRays
    BeginDrawPropeller  
    
    
    foreach surf $surfs {
	if {[ catch { .central.s process Mescape select surfaces "$surf" }] == 1} {
	    catch { .central.s process Mescape select surfaces "$surf" } 
	}
    }
    set progress [expr $progress + 1.0]
    foreach vol $vols {
	if {[ catch { GiD_Process Mescape select volumes "$vol" }] == 1} {
	    catch { GiD_Process Mescape select volumes "$vol" }
	}
    }
    
    catch { GiD_Process Mescape View Rotate Angle 0 0 }
    GiD_Process Mescape View Redraw
    
    GiD_Process Mescape Results AnalysisSel RANSOL $currStep 
    GiD_Process Mescape Results ContourFill Eff.Wake

    set progress 100
    
    GidUtils::EnableWarnLine
    if {$::debug != 1} {
	destroy $wakeBar
    }
    
}

proc wakeRes::picture {} {
    
    return
}

proc wakeRes::CreateRays {} {
    global CoordsCirc
    variable propPos 
    variable propDiam
    variable pi
    variable incrX
    variable progress

    set rad [expr 1.1*$propDiam*0.5]
    set count 0
    set CoordsCirc(rays,orig) [list [expr [lindex $propPos 0] + $incrX] [lindex $propPos 1] [lindex $propPos 2]]
    for {set ang -90} {$ang <= 90} {set ang [expr $ang + 30]} {
	set crX [expr [lindex $propPos 0] + $incrX] 
	set crY [expr [lindex $propPos 1] + $rad*cos($ang*$pi/180.0)]
	set crZ [expr [lindex $propPos 2] + $rad*sin($ang*$pi/180.0)]
	set CoordsCirc(rays,$count) [format "%.3g %.3g %.3g" $crX $crY $crZ]
	incr count
	set progress [expr $progress + 1.0]
    }
    set CoordsCirc(rays,total) $count
}

proc wakeRes::CreatePropeller {} {
    global CoordsCirc
    variable propPos 
    variable propDiam
    variable pi
    variable incrX
    variable progress

    set CoordsCircAux ""
    set circY ""
    set circZ ""
    set rad [expr $propDiam*0.5]
    set CoordsCirc(totalC) 0
    set CoordsCirc(totalR) 0
    set radii ""
    set count 0

    for {set r 0.3 } {$r <= 1.1 } {set r [expr $r + 0.1]} {
	lappend radii [expr $rad*$r*1.0]
	incr count
    }

    for {set j 0} {$j < [llength $radii]} {incr j} {
	for {set theta [expr -$pi*0.5]} {$theta <= [expr $pi*0.5]} {set theta [expr $theta + 0.005]} {
	    set cY [expr [lindex $propPos 1] + [lindex $radii $j]*cos($theta)]
	    set cZ [expr [lindex $propPos 2] + [lindex $radii $j]*sin($theta)]
	    set cX [expr [lindex $propPos 0] + $incrX] 
	    lappend CoordsCircAux [format "%.3g %.3g %.3g" $cX $cY $cZ]
	}
	for {set i 0} {$i < [llength $CoordsCircAux]} {incr i} {
	    set CoordsCirc(coords,$j,$i) [lindex $CoordsCircAux $i]
	    if {$j == 0} {
		incr CoordsCirc(totalC)
	    }
	}
	set CoordsCircAux ""
	incr CoordsCirc(totalR)
		set progress [expr $progress + 1.0]
    }
}
proc wakeRes::EndDrawPropeller { w } {
    global CoordsCirc

    set ret [DialogWin::messageBox -default ok -icon warning -message \
	    "This will end the wake results view.\nAre you sure to quit?" -parent $w \
	    -title "Close Wake Results" -type okcancel]
	if { $ret == "cancel" } { return }

    set surfs [GiD_Info postprocess get all_surfaces]
    set vols [GiD_Info postprocess get all_volumes]

    if { [info exists CoordsCirc(drawopengl)]} {
	catch { drawopengl unregister $CoordsCirc(drawopengl) }
	catch { unset CoordsCirc }
	::wakeRes::DeleteDrawList
	GiD_Process Mescape View Redraw
    }
    
    foreach surf $surfs {
	if {[ catch { .central.s process Mescape select surfaces "$surf" }]} {
	    catch { .central.s process Mescape select surfaces "$surf" } 
	}
    }
    foreach vol $vols {
	if {[ catch { GiD_Process Mescape select volumes "$vol" }]} {
	    catch { GiD_Process Mescape select volumes "$vol" }
	}
    }

    GiD_Process Mescape Results Geometry NoResults

    set cutsAll [GiD_Info postprocess get all_cuts]
    if {$cutsAll != ""} {
	GiD_Process Mescape Utilities Delete CutSets ALL_CUTSETS Yes
    }

#     foreach surf $surfs {
#         if {[regexp {S_Set_} $surf]} {
#             .central.s process Mescape Utilities Delete SurfaceSets $surf Yes
#         }
#     }

    GiD_Process Mescape View Zoom Frame
    
    bind $w <Destroy> ""
    destroy $w
}
proc wakeRes::DeleteDrawList { } {
    global CoordsCirc
    if { [info exists CoordsCirc(opengl_drawlist)] } {
	drawopengl draw -deletelists [list $CoordsCirc(opengl_drawlist) 1]
	catch { unset CoordsCirc(opengl_drawlist) }
    }
}
proc wakeRes::BeginDrawPropeller { } {
    global CoordsCirc

    catch { set CoordsCirc(drawopengl) [drawopengl register ::wakeRes::DrawPropeller]}
    GiD_Process Mescape View Redraw
}
proc wakeRes::DrawPropeller {} {
    global CoordsCirc
    variable propPos
    variable incrX
    variable incrY
    variable propDiam
    variable progress
    
    if { ![info exists CoordsCirc(opengl_drawlist)] } {
	GidUtils::WaitState .gid
	set CoordsCirc(opengl_drawlist) [drawopengl draw -genlists 1]
	drawopengl draw -newlist $CoordsCirc(opengl_drawlist) compile
	
	set color "0 0 0"
	
	for {set j 0} {$j < $CoordsCirc(totalR)} {incr j} {
	    drawopengl draw -color $color 
	    drawopengl draw -begin lineloop
	    for {set i 0} {$i < $CoordsCirc(totalC)} {incr i} {
		set cX [lindex $CoordsCirc(coords,$j,$i) 0]
		set cY [lindex $CoordsCirc(coords,$j,$i) 1]
		set cZ [lindex $CoordsCirc(coords,$j,$i) 2]
		drawopengl draw -vertex "$cX $cY $cZ"
	    }
	    drawopengl draw -end
	    set progress [expr $progress + 1.0]
	}
	
	set val 0
	for {set j 0} {$j < $CoordsCirc(rays,total)} {incr j} {
	    drawopengl draw -color $color 
	    drawopengl draw -begin lineloop
	    drawopengl draw -vertex "[lindex $CoordsCirc(rays,orig) 0] [lindex $CoordsCirc(rays,orig) 1] [lindex $CoordsCirc(rays,orig) 2]"
	    set crX [lindex $CoordsCirc(rays,$j) 0]
	    set crY [lindex $CoordsCirc(rays,$j) 1]
	    set crZ [lindex $CoordsCirc(rays,$j) 2]
	    drawopengl draw -vertex "$crX $crY $crZ"
	    drawopengl draw -end
	    drawopengl draw -color $color -rasterpos "$crX $crY $crZ"
	    drawopengl drawtext "$val"
	    set val [expr $val + 30]
	    set progress [expr $progress + 1.0]
	}
	
	set posX [expr [lindex $propPos 0] + $incrX]
	set posY [expr [lindex $propPos 1] - $incrY]
	foreach RR0 {0.3 0.7 1.1} { 
	    set posZ [expr [lindex $propPos 2] + $propDiam*0.5*$RR0]
	    drawopengl draw -color $color -rasterpos "$posX $posY $posZ"
	    drawopengl drawtext "R/R0 $RR0"
	    set progress [expr $progress + 1.0]
	}
     
	drawopengl draw -endlist
	GidUtils::EndWaitState .gid
    }
    drawopengl draw -call $CoordsCirc(opengl_drawlist)
    
}

proc wakeRes::GetVel {velFlag} {
    variable listVx
    variable currStep
    variable progress
    variable countE
    
    set resList ""
    set listVx ""
    
    set stepsAll [GiD_Info postprocess get all_steps "RANSOL"]
    set listL [llength $stepsAll]
    set currStep [lindex $stepsAll [expr $listL - 1]]
    
    set resList [GiD_Result get [list "Velocity_(m/s)" "RANSOL" "$currStep"]]
    
    if {$velFlag == 0} {
	set countE 0
	for {set i 2} {$i < [llength $resList]} {incr i} {
	    set aux2 [lindex $resList $i]
	    lappend listVx $aux2
	    incr countE
	    set progress [expr $progress + 0.0005] 
	}
    } else {
	for {set i $countE} {$i < [llength $resList]} {incr i} {
	    set aux2 [lindex $resList $i]
	    lappend listVx $aux2
	    set progress [expr $progress + 0.0005] 
	}
    }
    
    if {$velFlag == 1} {
	set surfs [GiD_Info postprocess get all_surfaces]
	foreach surf $surfs {
	    if {[regexp {S_Set_} $surf]} {
		.central.s process Mescape Utilities Delete SurfaceSets $surf Yes
	    }
	}
    }
    
    set resList ""

    return
    
}

proc wakeRes::CreateResW {} {
    variable listW
    variable currStep
    variable listVx
    variable flowVel
    variable progress
    variable propPos
    variable incrX
#     global Results

#     set I 0 
#################
# Estela Efectiva
################# 
    set where "OnNodes"
    set res "Result \"Eff.Wake\" \"RANSOL\" $currStep Scalar $where"
    set file [file join $::env(TEMP) tmpfile.flavia.res]
    set fout [open $file w]
    puts $fout "GiD Post Results File 1.0"
    puts $fout $res
    puts $fout "Values"
    foreach result $listVx {
	set V [lindex $result 1]
	set elem [lindex $result 0]
#         set val [.central.s info list_entities Elements $elem]
#         set Results($elem,nodes) [lindex $val 3]

	set We [expr ($flowVel - [lindex $V 3])/$flowVel]
	puts $fout "$elem $We"        

#         foreach node $Results($elem,nodes) {
#             set nodCoords [GiD_Info Coordinates $node] 
#             set nodCoords [lindex $nodCoords 0]
#             set nodX [lindex $nodCoords 0]
#             set nodY [lindex $nodCoords 1]
#             set nodZ [lindex $nodCoords 2]
#             
#             set Ri [expr sqrt(($nodY-[lindex $propPos 1])*($nodY-[lindex $propPos 1]) + \
#                     ($nodZ-[lindex $propPos 2])*($nodZ-[lindex $propPos 2]))]
#             set THi ""
#             if {$Ri > 0.0 } {
#                 set THi [expr acos(($nodY-[lindex $propPos 1])/$Ri)]
#             }
#             set listAux [list $I $node $Ri $THi $We]
#             set Results(res,$I) $listAux
#             set Results(countI) $I
#             incr I
#         }
		
	set progress [expr $progress + 0.0005]        
    }    
    puts $fout "End Values"

    close $fout
    GiD_Process Mescape files add $file
    
    set listVx ""
 
}

proc wakeRes::CalcNomWake {} {
    variable listW
    variable currStep
    variable listVx
    variable flowVel
    variable progress
    variable propPos
    variable incrX
    variable propDiam
    variable countE
    variable pi
    variable units
    global Results

    set count 0
    set I 0
    set currCuts [GiD_Info postprocess get cur_cuts]
    
    GiD_Process Mescape DoCut TurnIntoMeshes
    
    GetVel 1
    
    set incrA 0.0
    set angles ""
    for {set i 0} {$i <= 18} {incr i} {
	lappend angles $incrA
	set incrA [expr $incrA + 10.0]
    }

    set radii ""
    set rad [expr $propDiam*0.5]
    for {set r 0.3 } {$r <= 1.1 } {set r [expr $r + 0.1]} {
	lappend radii [expr $rad*$r*1.0]
	set Wn($count) 0.0
	incr count
    }
    
    foreach result $listVx {
	set V [lindex $result 1]
	set velN [expr abs([lindex $V 1])/[lindex $V 3]]
	set elem [lindex $result 0]
	set val [.central.s info list_entities Elements $elem]
	set Results($elem,nodes) [lindex $val 3]

	set We [expr ($flowVel - [lindex $V 3])/$flowVel]

	foreach node $Results($elem,nodes) {
	    set nodCoords [GiD_Info Coordinates $node] 
	    set nodCoords [lindex $nodCoords 0]
	    set nodX [lindex $nodCoords 0]
	    set nodY [lindex $nodCoords 1]
	    set nodZ [lindex $nodCoords 2]
	    
	    set Ri [expr sqrt(($nodY-[lindex $propPos 1])*($nodY-[lindex $propPos 1]) + \
		    ($nodZ-[lindex $propPos 2])*($nodZ-[lindex $propPos 2]))]
	    if {$Ri > 0.0 } {
		set THi [expr acos(($nodY-[lindex $propPos 1])/$Ri)]
		set indexR [expr [llength $radii] - 1]
		for {set j 0} {$j <= $indexR} {incr j} {
		    if {$Ri <= [lindex $radii $j]} {
		        for {set i $j} {$i <= $indexR} {incr i} {
		            set Wn($i) [expr $Wn($i) + $velN*$Ri*$Ri*0.5*$THi]
		        }
		    }
		}                              
	    }
	    set RR0 [expr $Ri/$rad]
	    if {$RR0 <= 1.1} {
		if {$THi == 0} {
		    set THi 0.0
		}
		set THi [expr $THi*180.0/$pi]
		foreach ang $angles {
		    set delta [expr abs($THi - $ang)]
		    if {$delta <= 0.1 } {
		        set listAux [list $THi $RR0 $We]
		        set Results(res,$I) $listAux
		        set Results(countI) $I
		        incr I
		    }
		}
	    }
	    
	    set progress [expr $progress + 0.0005] 
	}
    }    
    
    set listVx ""

    set projectPath [GiD_Info Project ModelName]
#     set initial [file rootname [file tail $projectPath]].xls
    set initial "EnsayoEstelaN_.xls"
    set ext ".xls"
    if { $::tcl_platform(platform) == "windows" } {
	set tofile [tk_getSaveFile -defaultextension $ext \
		-initialfile $initial -initialdir $projectPath \
		-title "Write Trasfer Functions"]
    } else {
	set tofile [Browser-ramR file save]
    }
    if { [file ext $tofile] != ".xls"} {
	WarnWin "Unknown extension for file '$tofile'"
	return 0
    }
    set file2 $tofile
    set res_file [open $file2 "w+"]

    set velKn [format "%.3g" [expr $flowVel*3600.0/1852.0]]
    if {$units == "\[m\]"} {
	set diamM [format "%.3g" [expr $propDiam*1.0]]
    } elseif {$units == "\[mm\]"} {
	set diamM [format "%.3g" [expr $propDiam*1.0e-3]]
    } elseif {$units == "\[cm\]"} {
	set diamM [format "%.3g" [expr $propDiam*1.0e-2]]
    }
 
    puts $res_file "\n\t RESULTADOS DE ESTELA SEGN ENSAYO N. "
    puts $res_file "\n\n CARENA N. \n CALADO MEDIO: (m) \n DIFERENCIA DE CALADOS:  (m) \n VELOCIDAD DEL BUQUE: $velKn (kn) \n DIMETRO DEL PROPULSOR: $diamM (m)"
    puts $res_file "\n\n\t ESTELA MEDIA RADIAL Y ESTELA NOMINAL"
    puts $res_file "\n Relacin de Radio \t Estela Media Radial \t Estela Nominal"
    
    set r 0.3
    for {set i 0 } {$i <= $indexR  } {incr i} {
	set fac [expr $pi*[lindex $radii $i]*[lindex $radii $i]]
	set Wn($i) [expr $Wn($i)/$fac]
	puts $res_file "$r \t  \t $Wn($i)"
	set r [expr $r + 0.1]
    }

    puts $res_file "\n\n\t ESTELA INTERPOLADA CORREGIDA"

    for {set j 0} {$j <= $Results(countI)} {incr j} {
	puts $res_file "\t [lindex $Results(res,$j) 0] \t [lindex $Results(res,$j) 1] \t [lindex $Results(res,$j) 2]"
    }
     
    close $res_file

}
