# tdyn2d.tcl  -*- TCL -*-
#---------------------------------------------------------------------------
#This file is written in TCL lenguage 
#For more information about TCL look at: http://www.sunlabs.com/research/tcl/
#
#At least two procs must be in this file:
#
#    InitGIDProject dir - Will be called whenever a project is begun to be used.
#           where dir is the project's directory
#    EndGIDProject - Will be called whenever a project ends to be used.
#
#For more information about GID internals, check the program scripts.
#---------------------------------------------------------------------------

namespace eval Tdyn {}

# Select 2D or 2DA problemtype
set ::2Dor2DA "2D"

############################################################################
#   InitGIDProject
############################################################################
proc InitGIDProject { dir } {
    global MenuNames MenuEntries MenuCommands MenuAcceler
    global MenuNamesP MenuEntriesP MenuCommandsP MenuAccelerP
    global VersionNumber ProgramName ProblemTypePriv GidPriv PropToPostPrev ProgNamePrev TdynPriv
    global ProgramTag GIDDEFAULT
    
    catch { unset TdynPriv }

    set TdynPriv(problemtypedir) $dir
    set ProblemTypePriv(problemtypedir) $dir

    # Saving previous problemtype
    if { [info exists ProgramName] } { 
	set ProgNamePrev $ProgramName
    } else {
	set ProgNamePrev ""
    }

    #
    #   Check GiD Version
    #
    set VersionRequired "8.0"
    set comp -1
    catch {
	set comp [GiDVersionCmp $VersionRequired]
    }
    if { $comp < 0 } {
	tk_dialogRAM .gid.tempwinw Warning \
		[= {Tdyn requires GiD %s or later} $VersionRequired] error 0 OK
    }
    #
    #   Define name and version and open splash window
    #
    if { $::tcl_platform(platform) == "windows" } {
	set exename "tdyn.exe"
    } else {
	set exename "tdyn"
    }
    set ExecFile [file join $dir "../scripts" $exename]

    set VersionNumberTmp [regsub -all {,} \
	    [eval exec [list $ExecFile] "-no_timetable -tool give_version"] {.}]
    set ProgramNameTmp Tdyn2D
    set NewProblemType "No"
    if { ![info exists ProgramName] } { 
	set ProgramName ""
    }
    if { ![info exists VersionNumber] } { 
	set VersionNumber ""
    }

    catch { unset ProblemTypePriv }
    catch { namespace delete TdynDefaults }
    set ProblemTypePriv(problemtypedir) $dir

    if { [string compare -nocase $ProgramName $ProgramNameTmp]!=0 || \
	    [string compare -nocase $VersionNumber $VersionNumberTmp]!=0 } {
	set ProgramName $ProgramNameTmp
	set VersionNumber $VersionNumberTmp
	Splash
	if { [winfo exists .splash] } {
	    tkwait window .splash
	}
	set NewProblemType "Yes"
    }   

    #
    #   Load Tdyn utilities file
    #
    lappend ::auto_path [file join $dir .. scripts] [file join $dir exec] [file join $GIDDEFAULT problemtypes common_lib]
    
    if { [catch {
	    package forget helpviewer
	} errormsg] } {
	WarnWin "[= {Error found}]: \n$errormsg."
	return
    }
       
    if { [catch {
	    if { [file exists [file join $dir .. scripts tdynutils.tcl]] } {
		source [file join $dir .. scripts tdynutils.tcl]
	    } else {
		if { [file exists [file join $dir .. scripts tdynutils.tbe]] } {
		    loadtbefile [file join $dir .. scripts tdynutils.tbe]
		} else {
		loadtbefile [file join $GIDDEFAULT problemtypes common_lib tdynutils.tbe]
		}
		loadtbefile [file join $dir .. scripts tdynutils.tbe]
	    }
    } errormsg] } {
	WarnWin "[= {Error found}]: [= {Cannot load file %s} [file join $dir .. scripts tdynutils.tcl]].\n\
		[= {Tdyn is not correctly installed}].\n$errormsg."
	return
    }

    
    if { [catch {
	    loadpackage [file join $dir .. scripts] base64
	    loadpackage [file join $dir .. scripts] helpviewer
	} errormsg] } {
	WarnWin "[= {Error found}]: [= {Tdyn is not correctly installed}].\n$errormsg."
	return
    }     
    #
    #   Configuracin para la primera vez que se abre Tdyn
    #
    if { [IsVarOnTdynDefaults first_start 1] } {
	WriteVarTdynDefaults first_start 0
	#ConfigureTdynWindow 0
	#tkwait window .gid.tdynconf
	#   Meshing surfaces with Rsurf
	.central.s process escape escape escape escape Utilities Variables \
		SurfaceMesher 1 escape escape escape
	#   Meshing surfaces with AdvancingfrontMapper
	.central.s process escape escape escape escape Utilities Variables \
		AdvancingfrontMapper 1 escape escape escape
	#   Meshing surfaces with smoothing
	.central.s process escape escape escape escape Utilities Variables \
		Highqualitysmoothing 2 escape escape escape
    }
    #
    #   Inicializacin de algunas variables necesarias
    #
    set ProblemTypePriv(check_functions) [IsVarOnTdynDefaults check_funct 1]
    set ProblemTypePriv(check_units) [IsVarOnTdynDefaults check_units 0]
    set ProblemTypePriv(edit_units) [IsVarOnTdynDefaults edit_units 1]
    set ProblemTypePriv(fluid_atype) [IsVarOnTdynDefaults fluid_atype 1]
    if { ($ProblemTypePriv(fluid_atype) ne "1") && ($ProblemTypePriv(fluid_atype) ne "2") \
	    && ($ProblemTypePriv(fluid_atype) ne "3") } {
	set ProblemTypePriv(fluid_atype) 1
    }
    set ProblemTypePriv(solid_atype) [IsVarOnTdynDefaults solid_atype 1]
    if { ($ProblemTypePriv(solid_atype) ne "1") && ($ProblemTypePriv(solid_atype) ne "2") \
	    && ($ProblemTypePriv(solid_atype) ne "3") } {
	set ProblemTypePriv(solid_atype) 1
    }
    if { [IsVarOnTdynDefaults show_naval 1]==1 } {
	WriteVarTdynDefaults show_naval 0
    }
    #
    #   Additional TCL Source Files
    #
    if { [catch {
	foreach i [list tkwizard tdynwizard tkwidget \
		materials mat.fluid mat.solid bond.fluid bond.solid \
		functioneditor interfaceutils init.datwin solv.datwin \
		prb.datwin unt.datwin tdynoutput] {
	    uplevel \#0 [list loadsource [file join $dir .. scripts] $i]
	}
    } errormsg] } {
	WarnWin "[= {Error found}]: [= {Tdyn is not correctly installed}].\n$errormsg."
	return
    }
    #
    #   Define los problemas que se resolvern (por defecto)
    #    
    set show_ransol [IsVarOnTdynDefaults show_ransol 1]
    set show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    set show_advect [IsVarOnTdynDefaults show_advect 1]
    set show_ursolver [IsVarOnTdynDefaults show_ursolver 1]
    set show_alemesh [IsVarOnTdynDefaults show_alemesh 1]
    set show_fluid [IsVarOnTdynDefaults show_fluid 1]
    set show_solid [IsVarOnTdynDefaults show_solid 1]
    set show_oddls [IsVarOnTdynDefaults show_oddls 1]
    #
    #   Checking pressure units (due to change of version)
    # 
    set punits [GiD_AccessValue get gendata Boundary_Pressure_Units]
    if { $punits eq {[m2/s2]} } {
	GiD_AccessValue set gendata Boundary_Pressure_Units {[Pa]}
    }

    set AnyToBeSolved 0
    if { $ProgramName == "Tdyn2D" } {
	set ProblemList [list Solve_FluidFlow_in_Fluid Solve_Temperature_in_Fluid \
		Solve_SpeciesConcentration_in_Fluid Solve_PhiProblems_in_Fluid \
		Solve_FluidFlow_in_Solid Solve_Temperature_in_Solid \
		Solve_SpeciesConcentration_in_Solid Solve_PhiProblems_in_Solid]
    } else {
	set ProblemList [list Solve_FluidFlow_in_Fluid Solve_Temperature_in_Fluid \
		Solve_SpeciesConcentration_in_Fluid Solve_PhiProblems_in_Fluid \
		Solve_FluidFlow_in_Solid Solve_Temperature_in_Solid \
		Solve_SpeciesConcentration_in_Solid Solve_PhiProblems_in_Solid \
		Solve_StructuralProblem_in_Solid]
    }
    set gendata [.central.s info gendata]
    foreach item $ProblemList {
	variable $item
	set $item [DWUnder2Space [join [lindex $gendata [expr [lsearch -regexp $gendata \
		[eval set nothing ^${item}(\$|\#CB)]] + 1]]]]
	if { [eval set nothing $$item] == "Yes" } {
	    set AnyToBeSolved 1
	    break
	}
    }
    if { $AnyToBeSolved == 0 } {
	if { $show_fluid } {
	    GiD_AccessValue set gendata Solve_Fluid Yes
	    if { $show_ransol } { 
		GiD_AccessValue set gendata Solve_FluidFlow_in_Fluid Yes
	    } elseif { $show_heatrans } { 
		GiD_AccessValue set gendata Solve_Temperature_in_Fluid Yes
	    } elseif { $show_advect } { 
		GiD_AccessValue set gendata Solve_SpeciesConcentration_in_Fluid Yes 
	    } elseif { $show_ursolver } { 
		GiD_AccessValue set gendata Solve_PhiProblems_in_Fluid Yes 
	    }
	} else {
	    GiD_AccessValue set gendata Solve_Solid Yes
	    if { $show_ransol } { 
		GiD_AccessValue set gendata Solve_FluidFlow_in_Solid Yes
	    } elseif { $show_heatrans } { 
		GiD_AccessValue set gendata Solve_Temperature_in_Solid Yes
	    } elseif { $show_advect } { 
		GiD_AccessValue set gendata Solve_SpeciesConcentration_in_Solid Yes
	    } elseif { $show_ursolver } { 
		GiD_AccessValue set gendata Solve_PhiProblems_in_Solid Yes
	    } elseif { $show_ramsolid } { 
		GiD_AccessValue set gendata Solve_StructuralProblem_in_Solid Yes
	    }
	}
    }
#
#   Actualizamos los menus de Tdyn
#
    UpdateTdynMenus
    UpdateTdynContextualMenu
#
#   Creates Bitmaps Toolbar
#
    MyPreTdyn2DBitmaps $dir 
    MyPostTdyn2DBitmaps $dir 
#
#   Pass materials as sets to Postprocess
#
    set ProblemTypePriv(PropertyToPost) [.central.s info variables PropertyToPost]
    #   Saving previous problemtype properties 
    set PropToPostPrev $ProblemTypePriv(PropertyToPost)

    if { $ProblemTypePriv(PropertyToPost) != "Material" } {
	.central.s process escape escape escape escape Utilities Variables\
		PropertyToPost Material escape escape escape
    }
#
#   Create menus and define the title of the window
#
    global GidPriv
    if { $::2Dor2DA == "2D" } {
	set GidPriv(ProgName) $ProgramName
    } else {
	set GidPriv(ProgName) ${ProgramName}A
    }
    ChangeWindowTitle
#
#   Open a tip
#
    set TipsWizPriv(show) [IsVarOnTdynDefaults showtips 1]
    if { $TipsWizPriv(show)==1 &&  $NewProblemType=="Yes" } {
	OpenTipsWizardWindow
    }
}

############################################################################
#   InitGIDPostProcess
############################################################################
proc InitGIDPostProcess {} {
}

############################################################################
#   Changed Language
############################################################################
proc ChangedLanguage { language } {
    global ProblemTypePriv
    # Update Menus
    UpdateTdynMenus
    # Update Tool Bar
    MyPreTdyn2DBitmaps $ProblemTypePriv(problemtypedir)
    MyPostTdyn2DBitmaps $ProblemTypePriv(problemtypedir)
}

############################################################################
#   EndGIDProject
############################################################################
proc EndGIDProject {} {
    # Close menu bar and define the original title of the window
    global ProblemTypePriv GidPriv PropToPostPrev ProgNamePrev
    catch {

    # Kill Bitmaps toolbar
    EndMyBitmaps

    # Kill tips windows (if exist)
    if { [winfo exists .gid.tipswizard] } {
	destroy .gid.tipswizard
    }
    # Kill forces and motions windows
    ForcesGraph::kill
    MotionsGraph::kill
    # Kill rest of windows (if exist)
    set winlist [list .gid.tdynconf .gid.fluidmat .gid.solidmat .gid.unw \
	    .gid.fluidbond .gid.solidbond .gid.contacts .gid.tdw .gid.tiw \
	    .gid.advoddls .gid.advturbu ]
    foreach winopen $winlist {
	if { [winfo exists $winopen] } {
	    destroy $winopen
	}      
    }

    #
    #   Set Gid properties to post, to previous problemtype value (pass LAYERS/MATERIALS as sets to Postprocess)
    #
	if {$ProgNamePrev != ""} {
	    set ProblemTypePriv(PropertyToPost) [.central.s info variables PropertyToPost]
	    if { $ProblemTypePriv(PropertyToPost) != $PropToPostPrev } {
		.central.s process escape escape escape escape Utilities Variables \
		    PropertyToPost $PropToPostPrev escape escape escape
	    }
	} else {
	    .central.s process escape escape escape escape Utilities Variables \
		PropertyToPost Layer escape escape escape
	}

    # Delete memory
    namespace delete initdatwin
    namespace delete solvdatwin
    namespace delete buttons
    namespace delete TUI_images
    namespace delete matfluid
    namespace delete matsolid
    namespace delete FEditor
    #namespace delete HelpViewer
    namespace delete TdynDrawGraph
    namespace delete ForcesGraph
    namespace delete MotionsGraph
    # Borramos el contenido de SelectGIDBatFile
    proc SelectGIDBatFile {} {
    }
    # Set Property to Post to the original value
    .central.s process escape escape escape escape Utilities Variables\
	    PropertyToPost $ProblemTypePriv(PropertyToPost) escape escape escape
    # Define the original title of the window
    set GidPriv(ProgName) GiD
    ChangeWindowTitle ""
    }
}

############################################################################
#   EndGIDPostprocess
############################################################################
proc EndGIDPostprocess {} {
}

############################################################################
#   Creamos las barras de herramientas de Tdyn
############################################################################
proc MyPreTdyn2DBitmaps { dir { type "DEFAULT INSIDETOP"} } {
    global MyPreBitmapsNames MyPreBitmapsCommands MyPreBitmapsHelp 
    global ProgramName ProblemTypePriv GidPriv

    set show_ransol [IsVarOnTdynDefaults show_ransol 1]
    set show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    set show_advect [IsVarOnTdynDefaults show_advect 1]
    set show_ursolver [IsVarOnTdynDefaults show_ursolver 1]
    set show_alemesh [IsVarOnTdynDefaults show_alemesh 1]
    set show_fluid [IsVarOnTdynDefaults show_fluid 1]
    set show_solid [IsVarOnTdynDefaults show_solid 1]
    set show_oddls [IsVarOnTdynDefaults show_oddls 1]

    set MyPreBitmapsNames(0) ""
    set MyPreBitmapsCommands(0) ""
    set MyPreBitmapsHelp(0) ""
    if { $show_ransol } { 
	lappend MyPreBitmapsNames(0) "images/fluidconstraints.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "RANSOL"]
	lappend MyPreBitmapsHelp(0) [= {Open fluid conditions window}]
    }
    if { $show_heatrans } { 
#        lappend MyPreBitmapsNames(0) "images/fire.gif" 
	lappend MyPreBitmapsNames(0) "images/thermo.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "HEATRANS"]
	lappend MyPreBitmapsHelp(0) [= {Open temperature conditions window}]
    }
    if { $show_advect } { 
	lappend MyPreBitmapsNames(0) "images/advect.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "ADVECT"]
	lappend MyPreBitmapsHelp(0) [= {Open species advection conditions window}]
    }
    if { $show_ursolver } { 
#        lappend MyPreBitmapsNames(0) "images/ursolver.gif" 
	lappend MyPreBitmapsNames(0) "images/phicap.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "URSOLVER"]
	lappend MyPreBitmapsHelp(0) [= {Open ursolver conditions window}]
    }
    if { $show_alemesh } { 
	lappend MyPreBitmapsNames(0) "images/alemesh.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "ALEMESH"]
	lappend MyPreBitmapsHelp(0) [= {Open mesh deformation conditions window}]
    } 
    if { $show_oddls } { 
	lappend MyPreBitmapsNames(0) "images/oddls.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "ODDLS"]
	lappend MyPreBitmapsHelp(0) [= {Open odd levelset conditions window}]
    }

    lappend MyPreBitmapsNames(0) "---" 
    lappend MyPreBitmapsCommands(0) ""
    lappend MyPreBitmapsHelp(0) ""

    if { $show_fluid } { 
	lappend MyPreBitmapsNames(0) "images/drop.gif" 
#        lappend MyPreBitmapsCommands(0) [list -np- GidOpenMaterials "MATERIALS_(FLUID)"]
	lappend MyPreBitmapsCommands(0) [list -np- materials::init "MATERIALS_(FLUID)"]
	lappend MyPreBitmapsHelp(0) [= {Open fluid materials window}]
    }
    if { $show_solid } { 
	lappend MyPreBitmapsNames(0) "images/column.gif" 
#        lappend MyPreBitmapsCommands(0) [list -np- GidOpenMaterials "MATERIALS_(SOLID)"]
	lappend MyPreBitmapsCommands(0) [list -np- materials::init "MATERIALS_(SOLID)"]
	lappend MyPreBitmapsHelp(0) [= {Open solid materials window}]
    }
    if { $show_fluid } { 
	lappend MyPreBitmapsNames(0) "images/fboundaries.gif" 
#         lappend MyPreBitmapsCommands(0) [list -np- GidOpenMaterials "BOUNDARIES_(FLUID)"]
	lappend MyPreBitmapsCommands(0) [list -np- materials::init "BOUNDARIES_(FLUID)"]
	lappend MyPreBitmapsHelp(0) [= {Open fluid boundaries window}]
    }
    if { $show_solid } { 
	lappend MyPreBitmapsNames(0) "images/sboundaries.gif" 
#         lappend MyPreBitmapsCommands(0) [list -np- GidOpenMaterials "BOUNDARIES_(SOLID)"]
	lappend MyPreBitmapsCommands(0) [list -np- materials::init "BOUNDARIES_(SOLID)"]
	lappend MyPreBitmapsHelp(0) [= {Open solid boundaries window}]
    } 
    lappend MyPreBitmapsNames(0) "images/contacts.gif"
    lappend MyPreBitmapsCommands(0) [list -np- materials::init "CONTACTS"]
    lappend MyPreBitmapsHelp(0) [= {Open contacts window}]

    lappend MyPreBitmapsNames(0) "---" 
    lappend MyPreBitmapsCommands(0) ""
    lappend MyPreBitmapsHelp(0) ""

    lappend MyPreBitmapsNames(0) "images/initial.gif" 
    lappend MyPreBitmapsCommands(0) [list -np- initdatwin::init .gid "all" ""]
    lappend MyPreBitmapsHelp(0) [= {Open initial and conditional data window}]

    lappend MyPreBitmapsNames(0) "---" 
    lappend MyPreBitmapsCommands(0) ""
    lappend MyPreBitmapsHelp(0) ""

    lappend MyPreBitmapsNames(0) "images/prdata.gif" 
#    lappend MyPreBitmapsCommands(0) [list -np- GidOpenProblemData "PROBLEM"]
    lappend MyPreBitmapsCommands(0) [list -np- prbdatwin::init .gid "all" ""]
    lappend MyPreBitmapsHelp(0) [= {Open general data window}]

    lappend MyPreBitmapsNames(0) "images/solvers.gif" 
#    lappend MyPreBitmapsCommands(0) [list -np- GidOpenProblemData "SOLVER"]
    lappend MyPreBitmapsCommands(0) [list -np- solvdatwin::init .gid "all" ""]
    lappend MyPreBitmapsHelp(0) [= {Open modules and solvers data window}]

    lappend MyPreBitmapsNames(0) "images/units.gif" 
#    lappend MyPreBitmapsCommands(0) [list -np- GidOpenProblemData "UNITS"]
    lappend MyPreBitmapsCommands(0) [list -np- unitdatwin::init .gid "all" ""]
    lappend MyPreBitmapsHelp(0) [= {Open units window}]

    lappend MyPreBitmapsNames(0) "---" 
    lappend MyPreBitmapsCommands(0) ""
    lappend MyPreBitmapsHelp(0) ""

    lappend MyPreBitmapsNames(0) "images/mesh.gif" 
    lappend MyPreBitmapsCommands(0) [list -np- MeshingWindow]
    lappend MyPreBitmapsHelp(0) [= {Start meshing window}]

    lappend MyPreBitmapsNames(0) "---" 
    lappend MyPreBitmapsCommands(0) ""
    lappend MyPreBitmapsHelp(0) ""

    lappend MyPreBitmapsNames(0) "images/calc.gif" 
    lappend MyPreBitmapsCommands(0) [list -np- StartCalculate local 0]
    lappend MyPreBitmapsHelp(0) [= {Launch calculation process}]

    lappend MyPreBitmapsNames(0) "images/info.gif" 
    lappend MyPreBitmapsCommands(0) [list -np- PWViewOutput]
    lappend MyPreBitmapsHelp(0) [= {View process info}]

    if { $show_ransol } {
	lappend MyPreBitmapsNames(0) "images/loads.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- OpenForcesWindow]
	lappend MyPreBitmapsHelp(0) [= {View forces on bodies}]
    }
    if { $show_ransol } {
	lappend MyPreBitmapsNames(0) "images/graphfor.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- ForcesGraph::init]
	lappend MyPreBitmapsHelp(0) [= {Graphs of forces on bodies}]
    }
    if { $show_alemesh } {
	lappend MyPreBitmapsNames(0) "images/graphmov.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- MotionsGraph::init]
	lappend MyPreBitmapsHelp(0) [= {Graphs of motions of bodies}]
    }
    lappend MyPreBitmapsNames(0) "images/stop.gif" 
    lappend MyPreBitmapsCommands(0) [list Utilities CancelProcess]
    lappend MyPreBitmapsHelp(0) [= {Cancel calculation process}]

    lappend MyPreBitmapsNames(0) "images/tdynicon24x24.gif" 
#     lappend MyPreBitmapsCommands(0) [list -np- Splash 0]
    lappend MyPreBitmapsCommands(0) [list -np- CreateWelcomeTDYN]
    if { $::2Dor2DA == "2D" } {
	lappend MyPreBitmapsHelp(0) [= "Tdyn 2D Interface"]
    } else {
	lappend MyPreBitmapsHelp(0) [= "Tdyn 2DA Interface"]
    }

    # prefix values:
    #          Pre        Only active in the preprocessor
    #          Post       Only active in the postprocessor
    #          PrePost    Active Always
    set prefix Pre
    if { $::2Dor2DA == "2D" } {
	set - [= "Tdyn2D pre toolbar"] ;# only for translations, to pick the sentence
	set ProblemTypePriv(pretoolbarwin) [CreateOtherBitmaps MyPreTdyn2DBar [= {Tdyn2D pre toolbar}] \
		MyPreBitmapsNames MyPreBitmapsCommands MyPreBitmapsHelp \
		$dir "MyPreTdyn2DBitmaps [list $dir]" $type $prefix]
	AddNewToolbar "$ProgramName prebar" ${prefix}MyBarTdyn2DWindowGeom "MyPreTdyn2DBitmaps [list $dir]"
    } else {
	set - [= "Tdyn2DA pre toolbar"] ;# only for translations, to pick the sentence
	set ProblemTypePriv(pretoolbarwin) [CreateOtherBitmaps MyPreTdyn2DABar [= {Tdyn2DA pre toolbar}] \
		MyPreBitmapsNames MyPreBitmapsCommands MyPreBitmapsHelp \
		$dir "MyPreTdyn2DBitmaps [list $dir]" $type $prefix]
	AddNewToolbar "${ProgramName}A prebar" ${prefix}MyBarTdyn2DAWindowGeom "MyPreTdyn2DBitmaps [list $dir]"
    }
}

proc MyPostTdyn2DBitmaps { dir { type "DEFAULT INSIDETOP"} } {
    global MyPostBitmapsNames MyPostBitmapsCommands MyPostBitmapsHelp 
    global ProgramName ProblemTypePriv GidPriv

    set show_ransol [IsVarOnTdynDefaults show_ransol 1]
    set show_alemesh [IsVarOnTdynDefaults show_alemesh 1]

    set MyPostBitmapsNames(0) ""
    set MyPostBitmapsCommands(0) ""
    set MyPostBitmapsHelp(0) ""

    lappend MyPostBitmapsNames(0) "images/info.gif" 
    lappend MyPostBitmapsCommands(0) [list -np- PWViewOutput]
    lappend MyPostBitmapsHelp(0) [= {View process info}]

    if { $show_ransol } {
	lappend MyPostBitmapsNames(0) "images/loads.gif" 
	lappend MyPostBitmapsCommands(0) [list -np- OpenForcesWindow]
	lappend MyPostBitmapsHelp(0) [= {View forces on bodies}]
    }
    if { $show_ransol } {
	lappend MyPostBitmapsNames(0) "images/graphfor.gif" 
	lappend MyPostBitmapsCommands(0) [list -np- ForcesGraph::init]
	lappend MyPostBitmapsHelp(0) [= {Graphs of forces on bodies}]
    }
    if { $show_ransol } {
	lappend MyPostBitmapsNames(0) "images/graphmov.gif" 
	lappend MyPostBitmapsCommands(0) [list -np- MotionsGraph::init]
	lappend MyPostBitmapsHelp(0) [= {Graphs of motions of bodies}]
    }
    lappend MyPostBitmapsNames(0) "images/tdynicon24x24.gif" 
#     lappend MyPostBitmapsCommands(0) [list -np- Splash 0]
    lappend MyPostBitmapsCommands(0) [list -np- CreateWelcomeTDYN]
    if { $::2Dor2DA == "2D" } {
	lappend MyPostBitmapsHelp(0) [= {Tdyn 2D Interface}]
    } else {
	lappend MyPostBitmapsHelp(0) [= {Tdyn 2DA Interface}]
    }

    # prefix values:
    #          Pre        Only active in the preprocessor
    #          Post       Only active in the postprocessor
    #          PrePost    Active Always
    set prefix Post
    if { $::2Dor2DA == "2D" } {
	set - [= {Tdyn2D post toolbar}] ;# only for translations, to pick the sentence
	set ProblemTypePriv(posttoolbarwin) [CreateOtherBitmaps MyPostTdyn2DBar [= {Tdyn2D post toolbar}] \
		MyPostBitmapsNames MyPostBitmapsCommands MyPostBitmapsHelp \
		$dir "MyPostTdyn2DBitmaps [list $dir]" $type $prefix]
	AddNewToolbar "$ProgramName postbar" ${prefix}MyBarTdyn2DWindowGeom "MyPostTdyn2DBitmaps [list $dir]"
    } else {
	set - [= {Tdyn2DA post toolbar}] ;# only for translations, to pick the sentence
	set ProblemTypePriv(posttoolbarwin) [CreateOtherBitmaps MyPostTdyn2DABar [= {Tdyn2DA post toolbar}] \
		MyPostBitmapsNames MyPostBitmapsCommands MyPostBitmapsHelp \
		$dir "MyPostTdyn2DBitmaps [list $dir]" $type $prefix]
	AddNewToolbar "${ProgramName}A postbar" ${prefix}MyBarTdyn2DAWindowGeom "MyPostTdyn2DBitmaps [list $dir]"
    }
}

proc EndMyBitmaps {} {
    global ProgramName ProblemTypePriv
    
    #
    # Remove Tdyn toolbars
    #  
    if { $::2Dor2DA == "2D" } {
	ReleaseToolbar "$ProgramName prebar"
    } else {
	ReleaseToolbar "${ProgramName}A prebar"
    }
    rename MyPreTdyn2DBitmaps ""
    catch { destroy $ProblemTypePriv(pretoolbarwin) }

    if { $::2Dor2DA == "2D" } {
	ReleaseToolbar "$ProgramName postbar"
    } else {
	ReleaseToolbar "${ProgramName}A postbar"
    }
    rename MyPostTdyn2DBitmaps ""
    catch { destroy $ProblemTypePriv(posttoolbarwin) }
}

############################################################################
#   Actualizamos los menus de Tdyn
############################################################################

proc UpdateTdynMenus { } {
    global VersionNumber ProgramName ProblemTypePriv GidPriv
    global MenuNames MenuEntries MenuCommands MenuAcceler MenuIcons
    global MenuNamesP MenuEntriesP MenuCommandsP MenuAccelerP MenuIconsP

    global programTag GIDDEFAULT

    set dir $ProblemTypePriv(problemtypedir)
#
#   Oculta Books y Titles de conditiones no aplicables
#
    set show_ransol [IsVarOnTdynDefaults show_ransol 1]
    set show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    set show_advect [IsVarOnTdynDefaults show_advect 1]
    set show_ursolver [IsVarOnTdynDefaults show_ursolver 1]
    set show_alemesh [IsVarOnTdynDefaults show_alemesh 1]
    set show_fluid [IsVarOnTdynDefaults show_fluid 1]
    set show_solid [IsVarOnTdynDefaults show_solid 1]
    set show_oddls [IsVarOnTdynDefaults show_oddls 1]

    if { !$show_ransol } { 
	GidShowBook conditions RANSOL 0
    }
    if { !$show_heatrans } { 
	GidShowBook conditions HEATRANS 0
    }
    if { !$show_advect } { 
	GidShowBook conditions ADVECT 0
    }
    if { !$show_ursolver } { 
	GidShowBook conditions URSOLVER 0
    }
    if { !$show_alemesh } { 
	GidShowBook conditions ALEMESH 0
    }
    if { !$show_oddls } { 
	GidShowBook conditions ODDLS 0
    }
    if { !$show_fluid } { 
	GidShowBook materials MATERIALS_(FLUID) 0
	GidShowBook materials BOUNDARIES_(FLUID) 0
    }
    if { !$show_solid } { 
	GidShowBook materials MATERIALS_(SOLID) 0
	GidShowBook materials BOUNDARIES_(SOLID) 0
    }

#
#   Add new entries in Utilities Pulldown
#    
    set ipos [lsearch $MenuNames [_ {Utilities}]]
    if { $ipos != -1 } {
	#AppendMenuOptionPre $ipos ---
	#AppendMenuOptionPre $ipos "[= {Tdyn Report}]..." {-np- UseCreateReportWindow}
	if { $show_ransol || $show_alemesh } { 
	    AppendMenuOptionPre $ipos  --- ""
	}
	if { $show_ransol } {
	    AppendMenuOptionPre $ipos  [= "Forces on boundaries"] {-np- OpenForcesWindow}
	    AppendMenuOptionPre $ipos  [= "Forces graph"] {-np- ForcesGraph::init}
	}
	if { $show_alemesh } {
	    AppendMenuOptionPre $ipos  [= "Motions graph"] {-np- MotionsGraph::init}
	}
    }
    InsertMenuOption Utilities "[= {Tdyn Preferences}]..." 3 ConfigureTdynWindow \
	    PRE insert
#
#   Add new entries in Help Preprocess Pulldown
#    
    set ipos [lsearch $MenuNames [_ {Help}]]
    if { $ipos != -1 } {
	if { $::2Dor2DA == "2D" } {
	  if { [catch {loadpackage [file join $GIDDEFAULT problemtypes common_lib] gid_groups_conds 1.0
		} errormsg] } {
		InsertMenuOption Help "[= {Tdyn help}]..." 0 [list TdynHelpWindow reference] \
		    PREPOST insert       
	  } else {
		InsertMenuOption Help "[= {Tdyn help}]..." 0 [list Tdyn::Help reference] \
		    PREPOST insert
	    }
	AppendMenuOptionPre $ipos [= "About Tdyn2D..."] [list -np- CreateWelcomeTDYN ]
	} else {
	    if { [catch {loadpackage [file join $GIDDEFAULT problemtypes common_lib] gid_groups_conds 1.0
		} errormsg] } { 
		InsertMenuOption Help "[= {Tdyn help}]..." 0 [list Tdyn::Help reference] \
		PREPOST insert
	} else {
	    InsertMenuOption Help "[= {Tdyn help}]..." 0 [list Tdyn::Help reference] \
		    PREPOST insert                  
	}
	AppendMenuOptionPre $ipos [= "About Tdyn2DA..."] [list -np- CreateWelcomeTDYN ]
    }
	RemoveMenuOption Help "[= {Tutorials}]..." PREPOST
	AppendMenuOptionPre $ipos ---
	AppendMenuOptionPre $ipos [= "Tip of the day..."] [list -np- OpenTipsWizardWindow]
    }
    
#
#   Add new entries in Help Postprocess Pulldown
#

    set ipos [lsearch $MenuNames [_ {Help}]]
    if { $ipos != -1 } {
	if { $::2Dor2DA == "2D" } {
	    AppendMenuOptionPost $ipos [= "About Tdyn2D..."] [list -np- CreateWelcomeTDYN ]
	} else {
	    AppendMenuOptionPost $ipos [= "About Tdyn2DA..."] [list -np- CreateWelcomeTDYN ]
	}
	AppendMenuOptionPost $ipos ---
	AppendMenuOptionPost $ipos [= "Tip of the day..."] [list -np- OpenTipsWizardWindow]
	
    }

#
#   Change entries in Meshing menu
#
    set meshname [_ {Meshing}]
    set ipos [lsearch $MenuNames $meshname]
    if { $ipos == -1 } {
	set meshname [_ {Mesh}]
	set ipos [lsearch $MenuNames $meshname]
    }
    if { $ipos != -1 } {  
	set jpos [lsearch $MenuEntries($ipos) "[_ {Generate}]..."]
	if { $jpos == -1 } {
	    set jpos [lsearch $MenuEntries($ipos) "[_ {Generate mesh}]..."]
	}
	if { $jpos != -1 } {
	    set MenuCommands($ipos) [lreplace $MenuCommands($ipos) $jpos $jpos "-np- MeshingWindow"]
	}

	lappend MenuEntries($ipos) "---"
	lappend MenuCommands($ipos) ""
	lappend MenuAccelerP($ipos) ""
	
	lappend MenuEntries($ipos) "[= {Scale assigned sizes}]..."
	lappend MenuCommands($ipos) [list -np- ScaleMeshSizes]
	lappend MenuAcceler($ipos) ""
	
	catch {
	    RemoveMenuOption $meshname [_ {Element type}]>---  PRE
	    RemoveMenuOption $meshname [_ {Element type}]>---  PRE
	    RemoveMenuOption $meshname [_ {Element type}]>[_ {Tetrahedra}]  PRE
	    RemoveMenuOption $meshname [_ {Element type}]>[_ {Hexahedra}]   PRE
	    RemoveMenuOption $meshname [_ {Element type}]>[_ {Prism}] PRE
	    RemoveMenuOption $meshname [_ {Element type}]>---  PRE
	    RemoveMenuOption $meshname [_ {Element type}]>[_ {Only points}] PRE
	    RemoveMenuOption $meshname [_ {Element type}]>[_ {Sphere}] PRE
	}
    }
#
#   Change entries in Data menu
#
    GidChangeDataLabel {Data units} ""
    GidChangeDataLabel {Interval} ""
    GidChangeDataLabel {Conditions} ""
    GidChangeDataLabel {Materials} ""
    GidChangeDataLabel {Interval Data} ""
    GidChangeDataLabel {Problem Data} ""

    GidAddUserDataOptionsMenu [= {Conditions}] "TdynConditionsSubmenu %W" 2
    GidAddUserDataOptions "---" "" 3
    GidAddUserDataOptionsMenu [= {Materials}] "TdynMaterialsSubmenu %W" 4
    GidAddUserDataOptionsMenu [= {Boundaries}] "TdynBoundariesSubmenu %W" 5
    GidAddUserDataOptions "---" "" 6
    GidAddUserDataOptions     [= {Initial Data}] {initdatwin::init .gid "all" ""} 7
    GidAddUserDataOptions "---" "" 8
    GidAddUserDataOptions     [= {Problem Data}] {prbdatwin::init .gid "all" ""} 9
    GidAddUserDataOptions     [= {Modules Data}] {solvdatwin::init .gid "all" ""} 10
    GidAddUserDataOptions     [= {Units Data}] {unitdatwin::init .gid "all" ""} 11
    GidAddUserDataOptions "---" "" 12
#    if { [IsVarOnTdynDefaults show_oddls 1] } {
#        GidAddUserDataOptionsMenu [= {Problem Wizards}] "TdynWizardsSubmenu %W" 13
#        GidAddUserDataOptions "---" "" 14
#    }
#
#   Add a new entries in View results, Utilities and Calculate menu
#
    set idx [lsearch $MenuNames [_ {Calculate}]]
    if { $idx != -1 } {
	set i 0
	InsertMenuOption [_ Calculate] [_ Calculate] $i \
		[list StartCalculate local 0] PRE replace
#        InsertMenuOption [_ Calculate] [_ "Calculate and quit"] [incr i] \
#            [list StartCalculate local 1] PRE replace
	InsertMenuOption [_ Calculate] [_ "Calculate remote"] [incr i] \
		[list StartCalculate remote 0] PRE replace
	set i 5
	InsertMenuOption [_ Calculate] --- [incr i] "" PRE
	InsertMenuOption [_ Calculate] [= "Cancel results"] [incr i] \
		TdynCancelResults PRE
	
    } else {
	WarnWin "[= {Tdyn version and GiD version not compatible}]."
    }

    set ipos [lsearch $MenuNamesP [_ {View results}]]
    if { $ipos != -1 } {
	AppendMenuOptionPost $ipos ---
	AppendMenuOptionPost $ipos [= {Time Table}] {-np- OpenTimeTableWindow}

	if { $show_ransol || $show_alemesh } {
	    AppendMenuOptionPost $ipos ---
	}
	if { $show_ransol } {
	    AppendMenuOptionPost $ipos [= {Forces on Boundaries}] {-np- OpenForcesWindow}
	}
	if { $show_ransol } {
	    AppendMenuOptionPost $ipos [= {Forces Graph}] {-np- ForcesGraph::init}
	}
	if { $show_alemesh } {
	    AppendMenuOptionPost $ipos [= {Motions Graph}] {-np- MotionsGraph::init}
	}
    } else {
	WarnWin "[= {Tdyn version and GiD version not compatible}]."
    }  

    set idx [lsearch $MenuNamesP [_ {Utilities}]]
    if { $idx != -1 } {
	#AppendMenuOptionPost $ipos ---
	#AppendMenuOptionPost $ipos "[= {Tdyn Report}]..." {-np- UseCreateReportWindow}
    } else {
	WarnWin "[= {Tdyn version and GiD version not compatible}]."
    }

    UpdateMenus
#    CreateTopMenus
}

proc TdynHelpWindow { args } {

    global ProblemTypePriv

    set dir $ProblemTypePriv(problemtypedir)
    
    package require helpviewer
    
    set lan [GidUtils::GetCurrentLanguage]
    if { ![file exists [file join $dir .. info_ramseries $lan]] } {
	set lan es
    }
    set file [file join $dir .. info_tdyn $lan tdynreference_toc.html]
    HelpViewer::HelpWindow $file
} 

proc AppendMenuOptionPre { ipos entry { command "" } { acceler "" } { icon "" } } {
    global MenuNames MenuEntries MenuCommands MenuAcceler MenuIcons
    
    lappend MenuEntries($ipos) $entry
    lappend MenuCommands($ipos) $command
    lappend MenuAcceler($ipos) $acceler
    lappend MenuIcons($ipos) $icon
}

proc AppendMenuOptionPost { ipos entry { command "" } { acceler "" } { icon "" } } {
    global MenuNamesP MenuEntriesP MenuCommandsP MenuAccelerP MenuIconsP
    
    lappend MenuEntriesP($ipos) $entry
    lappend MenuCommandsP($ipos) $command
    lappend MenuAccelerP($ipos) $acceler
    lappend MenuIconsP($ipos) $icon
}

proc UpdateTdynContextualMenu { } {

    catch {
	ContextualEntity::RegisterUserDataCommand StartTdynContextualMenu
    }
}

#
#  Submenus of Menu Data
#
proc TdynConditionsSubmenu { w } {
    $w del 0 end
    if { [IsVarOnTdynDefaults show_ransol 1] } { 
	$w add command -label [= {Ransol}] -command [list GidOpenConditions RANSOL]
    }
    if { [IsVarOnTdynDefaults show_heatrans 1] } { 
	$w add command -label [= {Heatrans}] -command [list GidOpenConditions HEATRANS]
    }
    if { [IsVarOnTdynDefaults show_advect 1] } { 
	$w add command -label [= {Advect}] -command [list GidOpenConditions ADVECT]
    }
    if { [IsVarOnTdynDefaults show_ursolver 1] } { 
	$w add command -label [= {Ursolver}] -command [list GidOpenConditions URSOLVER]
    } 
    if { [IsVarOnTdynDefaults show_naval 1] } { 
	$w add command -label [= {Naval}] -command [list GidOpenConditions NAVAL]
    }
    if { [IsVarOnTdynDefaults show_alemesh 1] } { 
	$w add command -label [= {Alemesh}] -command [list GidOpenConditions ALEMESH]
    }
    if { [IsVarOnTdynDefaults show_oddls 1] } { 
	$w add command -label [= {OddLS}] -command [list GidOpenConditions ODDLS]
    } 
    $w configure -postcommand ""
}
proc TdynMaterialsSubmenu { w } {
    $w del 0 end
    if { [IsVarOnTdynDefaults show_fluid 1] } { 
	$w add command -label [= {Fluid}] -command "materials::init MATERIALS_(FLUID)"
    }
    if { [IsVarOnTdynDefaults show_solid 1] } { 
	$w add command -label [= {Solid}] -command "materials::init MATERIALS_(SOLID)"
    }
    $w configure -postcommand ""
}
proc TdynBoundariesSubmenu { w } {
    $w del 0 end
    if { [IsVarOnTdynDefaults show_fluid 1] } {
	$w add command -label [= {Fluid}] -command "materials::init BOUNDARIES_(FLUID)"
    }
    if { [IsVarOnTdynDefaults show_solid 1] } { 
	$w add command -label [= {Solid}] -command "materials::init BOUNDARIES_(SOLID)"
    }
    if { [IsVarOnTdynDefaults show_solid 1] && [IsVarOnTdynDefaults show_fluid 1] && \
	([IsVarOnTdynDefaults show_heatrans 1] || [IsVarOnTdynDefaults show_advect 1] || \
	[IsVarOnTdynDefaults show_ursolver 1]) } { 
	$w add command -label [= {Contact}] -command "materials::init CONTACTS"
    }
    $w configure -postcommand ""
}
proc TdynWizardsSubmenu { w } {
    $w del 0 end
    if { [IsVarOnTdynDefaults show_oddls 1] } {
#        $w add command -label [= {ODDLS...}] -command "OpenODDLSWizardWindow"
	$w add command -label [= {ODDLS...}] -command "WarnWin \"[= {Sorry, but this option is not available}]\""
    }
    $w configure -postcommand ""
}

# Save GiD project files (save additional files in the new directory)
proc SaveGIDProject { filespd } {
#    set dirinfo [lindex [.central.s info Project] 1]
#    set rootname [lindex [file split $dirinfo] end]
#    set from [file join [lindex [.central.s info Project] 1].gid $rootname.flavia.rst]
#    set to [file rootname $filespd].flavia.rst
#    WarnWin "Saving $to from $from"
}

#
#   Assign Mesh Criteria to Lines and Generate the Mesh
#
proc MeshingWindow {} {
    global ProblemTypePriv
    global scaledSize 
    
    set auxScale [info exists scaledSize]
    if {$auxScale == 0} {
	set scaledSize 0.0
    }


    if { [lsearch [image names] back-16] == -1 } {
	image create photo back-16 -data {
	    R0lGODlhEAAQAIUAAPwCBBRSdBRObCQ2TBxObISevAQCBNzu/BRGZPz6/FzC
	    3Pz+/HTS5ByyzJze7Mzq9ITC3AQWLAyWvBSavFyuxAwaLAwSHBRafBSOrDzW
	    5AyixCS61ETW3CzG1AQeLAweLAxefBSStEze7CSWtCyatBSCnBRWfAwmPBRW
	    dByixAQSHAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAAAALAAAAAAQABAAAAZi
	    QIBwSCwah4HjUTBQFgkFg3MoKBykU0QhoUAIAuAksbpgNByPxQMSGVsVDYlk
	    IqdUiJYLJqORbDgcHRseRR8gISIaEyMkGCVYRBEmeyAnlgaQkSgpmU4RAZ1O
	    KqFOpFNGfkEAIf5oQ3JlYXRlZCBieSBCTVBUb0dJRiBQcm8gdmVyc2lvbiAy
	    LjUNCqkgRGV2ZWxDb3IgMTk5NywxOTk4LiBBbGwgcmlnaHRzIHJlc2VydmVk
	    Lg0KaHR0cDovL3d3dy5kZXZlbGNvci5jb20AOw==
	}
	image create photo forward-16 -data {
	    R0lGODlhEAAQAIUAAPwCBAwyTBRObAw2VDR+nCRKZOzy/KTe7Pz+/KTK3Nzu
	    /Lze7FS+1AyexAyuzBSavAyOtBSmzOTy/BRqjNTm9IzO5ETS3ETa5By61Ayi
	    xByixBRmjAQGDBxCXGSivCySrCSWtBTC3AQOHAQWHAxWdEze7AQKFBRCXAwq
	    PAQCBAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAAAALAAAAAAQABAAAAZj
	    QIBwSCwahYGjUjBQGgWEpHNYMBCaT4G2UDggos+EwmBYMBpf6VBgYDgeEMgj
	    IpmoAQVKxXLBPDIXGhscRB0eHyAgDSGBGyJFASMiIiMkJYImUwAnmJqbjp4A
	    KCmhAKSlTn5BACH+aENyZWF0ZWQgYnkgQk1QVG9HSUYgUHJvIHZlcnNpb24g
	    Mi41DQqpIERldmVsQ29yIDE5OTcsMTk5OC4gQWxsIHJpZ2h0cyByZXNlcnZl
	    ZC4NCmh0dHA6Ly93d3cuZGV2ZWxjb3IuY29tADs=
	}
    }

    set w .gid.meshing

    if { [winfo exists $w] } return
    if { [info procs InitWindow] != "" } {
	InitWindow $w [= {Meshing Window}] PreMeshingWindowGeom ""
    } else {
	toplevel $w
    }

    set ProjectStatus [.central.s info list_entities status]
    set ProblemTypePriv(sizetransit) 0.0 
    set ProblemTypePriv(genesize) 0.0
    regexp -line "Unstructured size transition\:[ ]?(.+)$" $ProjectStatus -> ProblemTypePriv(sizetransit)
    regexp -line "Last size used\:[ ]?(.+)$" $ProjectStatus -> ProblemTypePriv(genesize)
    if {![string is double -strict $ProblemTypePriv(genesize)] | $ProblemTypePriv(sizetransit) <= 0.0} {
	set ProblemTypePriv(sizetransit) [.central.s info variable SizeTransitionsFactor]
    }
    if {![string is double -strict $ProblemTypePriv(genesize)] | $ProblemTypePriv(genesize) <= 0.0} {
	if {![string is double -strict $ProblemTypePriv(genesize)] | $ProblemTypePriv(genesize) <= 0.0} {
	    set ProblemTypePriv(genesize) [format {%.3g} [expr 0.1*[LayerLength "" D]]]
	    if { $ProblemTypePriv(genesize) == 0 } {
		set ProblemTypePriv(genesize) 1.0
	    }
	}
    }
    if { $scaledSize != 0.0 } {
	set ProblemTypePriv(genesize) [format {%.3g} $scaledSize]
    }
    set ProblemTypePriv(genesize) [string trim $ProblemTypePriv(genesize)]
    if { ![info exist ProblemTypePriv(meshingtype)] } {
	set ProblemTypePriv(meshingtype) meshall
    }

    labelframe $w.f0 -text [= "Unstructured size transitions"]

    set f01 [frame $w.f0.f1]
    label $f01.l1 -text [= slow] -anchor nw
    label $f01.l2 -text [= fast] -anchor ne
    grid $f01.l1 $f01.l2 -sticky new -padx 5
    grid columnconfigure $f01 "0 1" -weight 1

    set f02 [frame $w.f0.f2]
    button $f02.bt0 -image back-16 -bd 1 -relief flat -overrelief raised \
	    -command {set ProblemTypePriv(sizetransit) [expr $ProblemTypePriv(sizetransit)-0.1]}
    scale $f02.sc -orient h -from 0.1 -to 1.0 \
	    -resolution 0.1 -variable ProblemTypePriv(sizetransit)
    button $f02.bt1 -image forward-16  -bd 1 -relief flat -overrelief raised \
	    -command {set ProblemTypePriv(sizetransit) [expr $ProblemTypePriv(sizetransit)+0.1]}
    grid $f02.bt0 $f02.sc $f02.bt1 -sticky new
    grid columnconfigure $f02 1 -weight 1

    grid $f01     -     -sticky nwe -padx 2 -pady 2
    grid $f02     -     -sticky nsew -padx 2 -pady 2
    grid columnconfigure $w.f0 1 -weight 1
    grid rowconfigure $w.f0 2 -weight 1

    GidHelp "$w.f0" "[= {It controls whether the transitions between\
	    different element sizes are slow or fast}]."

    frame $w.f1 -relief ridge -bd 2
    radiobutton $w.f1.r1 -text [= {Mesh all lines}] \
	    -var ProblemTypePriv(meshingtype) -value meshall
    radiobutton $w.f1.r2 -text [= {Mesh lines in active layers}] \
	    -var ProblemTypePriv(meshingtype) -value meshlayers
    radiobutton $w.f1.r3 -text [= {Mesh using default options}] \
	    -var ProblemTypePriv(meshingtype) -value meshdefault
    
    GidHelp "$w.f1" "[= {Select what line entities will be meshed}]."

    grid $w.f1.r1 -sticky w
    grid $w.f1.r2 -sticky w
    grid $w.f1.r3 -sticky w
    grid columnconf $w.f1 0 -weight 1

    set ProblemTypePriv(deletefiles) yes
    frame $w.fa -relief ridge -bd 2
    checkbutton $w.fa.c1 -text [= {Delete result files}] \
	    -var ProblemTypePriv(deletefiles) -onvalue yes -offvalue no

    GidHelp "$w.fa" "[= {Select to delete existing result files}]."

    grid $w.fa.c1 -sticky w
    grid columnconf $w.fa 0 -weight 1

    frame $w.f2 -relief ridge -bd 2
    label $w.f2.lg -text [= {Maximum element size}]
    entry $w.f2.eg -relief sunken -textvariable ProblemTypePriv(genesize) -width 2  

    GidHelp "$w.f2" "[= {Insert a value for the global element size}]."

    grid $w.f2.lg $w.f2.eg -sticky w -padx 2 -pady 2
    grid conf $w.f2.eg -sticky we
    grid columnconf $w.f2 1 -weight 1

    grid $w.f2 -sticky we -padx 3 -pady 3
    grid $w.f1 -sticky we -padx 3 -pady 3
    grid $w.fa -sticky we -padx 3 -pady 3
    grid $w.f0 -sticky we -padx 3 -pady 3

    frame $w.buts
    button $w.buts.gn -text [= {Generate}] -width 10 -bd 1 -command  "MeshingCriteria $w"
    button $w.buts.cn -text [= {Cancel}] -width 10 -bd 1 -command "destroy $w"
    button $w.buts.hp -image [imagefromtdyn questionarrow.gif] -command "PickHelp $w.buts.hp" \
	    -highlightthickness 0 -bd 1 -relief flat -overrelief raised
    pack $w.buts.gn $w.buts.cn $w.buts.hp -side left -padx 5 -ipadx 5 -pady 5 

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $w.buts.cn"
    bind $w <Escape> "tkButtonInvoke $w.buts.cn"
    bind $w <Return> "tkButtonInvoke $w.buts.gn"
    grid $w.buts -sticky s

    grid rowconfigure $w {0 1 2} -weight 1
    grid columnconfigure $w 0 -weight 1
    wm geom $w ""
}

proc MeshingCriteria {w} {
    global ProblemTypePriv
    #   Controles iniciales
    if {![IsWord real+ $ProblemTypePriv(genesize) $w "[= {In Elements general size field}]."] } {
	return 
    } else {
	if { $ProblemTypePriv(genesize) == 0.0 } {
	    tk_dialogRAM .gid.tempwin [= {Error window}] "[= {General size must be greater than 0.0}]." \
		    error 0 OK
	    return 
	}
	destroy $w
    }    
    #   Se desabilitan ventanas y grficos
    .central.s disable windows 1 
    .central.s disable graphinput 1 
    #   Se cancela la malla que pueda existir
    .central.s process escape escape escape escape Meshing CancelMesh Yes
    # Se borran los archivos de resultados
    if { $ProblemTypePriv(deletefiles) == "yes" } {
	set resfiles [list flavia flavia.res flavia.war flavia.inf flavia.tim \
		flavia.out flavia.ram.res flavia.ram.msh flavia.sat flavia.for \
		flavia.mov]
	DeleteFiles $resfiles
    }
    #   Se aplica la transicin de tamaos
    set ProblemTypePriv(oldsizetransit) [.central.s info variable SizeTransitionsFactor]
   .central.s process escape escape escape escape utilities variables \
	    SizeTransitionsFactor $ProblemTypePriv(sizetransit) escape escape
    #   Se definen las capas que se mallarn
    .central.s process escape escape escape escape \
	    Meshing MeshCriteria Mesh Surfaces 1: escape
    case $ProblemTypePriv(meshingtype) {
	meshall {
	    .central.s process escape escape escape escape \
		    Meshing MeshCriteria Mesh Lines 1: escape
	}
	meshlayers  { 
	    set layers_on_list [.central.s info layers -on]
	    foreach curr_layer [lrange $layers_on_list 0 end] {
		.central.s process escape escape escape escape \
		        Meshing MeshCriteria Mesh Lines layer:$curr_layer escape
	    }
	
	}
    }
    #   Se genera la malla
    .central.s disable windows 0
    .central.s process escape escape escape escape Meshing Generate $ProblemTypePriv(genesize)
    #   Se ha acabado todo (se mantiene la transicin de tamaos)
    .central.s process escape escape escape escape utilities variables \
	    SizeTransitionsFactor $ProblemTypePriv(oldsizetransit) escape escape
    .central.s disable graphinput 0
}

#
#  Procedure to assign Mesh Criteria to Lines
#
proc AssignMeshCriteriaToLines {} {

    set answer [tk_messageBox -message [= {Assign <Mesh criteria> to all lines?}] \
		    -title [= {Assign <Mesh criteria>}]  -type yesnocancel -icon question]


    case $answer {
	yes {
		    .central.s process escape escape escape escape \
		    Meshing MeshCriteria Mesh Lines 1: escape
	}
	no  {
	}
	cancel return
    }
}

proc Splash { { autodestroy 1 } } {
    global GIDDEFAULT VersionNumber ProgramName ProblemTypePriv
    
    if { [.gid.central.s disable windows] } { return }
    
    if { [ winfo exist .splash]} {
	destroy .splash
	update
    }
    
    if { [catch {         
	    toplevel .splash
	    wm withdraw .splash
	    
	    set dir $ProblemTypePriv(problemtypedir)
	    set im [image create photo -file [ file join $dir images/tdyn-open.gif]]
	    set x [expr [winfo screenwidth .splash]/2-[image width $im]/2]
	    set y [expr [winfo screenheight .splash]/2-[image height $im]/2]
	    
	    wm geom .splash +$x+$y
	    wm transient .splash .gid
	    wm overrideredirect .splash 1
	    pack [label .splash.l -image $im -relief ridge -bd 2]    
	    if { $::2Dor2DA == "2D" } {
		label .splash.lv -text [= {%s: Ver. %s} $ProgramName $VersionNumber] \
		-relief sunken -bg #9ab6ce -font "times 14"
	    } else {
		label .splash.lv -text [= {%s: Ver. %s} ${ProgramName}A $VersionNumber] \
		-relief sunken -bg #9ab6ce -font "times 14"
	    }
	    place .splash.lv -x 35 -y 270
	    bind .splash <1> "destroy .splash"
	    bind .splash <KeyPress> "destroy .splash"
	    grab .splash
	    focus .splash
	    update
	    wm deiconify .splash
	    raise .splash .gid
	    update
	    if { $autodestroy } {
		after 2000 "if { [ winfo exist .splash] } { 
		destroy .splash
		}"
	    }
	}] } {
    if { [ winfo exist .splash]} {
	destroy .splash
	update
    }
}
}


# #
# #  Procedure to create Tdyn2D About window
# #
# proc AboutTdyn2D { dir { overw .gid } } {
#     global GIDDEFAULT VersionNumber
# 
#     if { $::2Dor2DA == "2D" } {
#         set ProgName [= {Tdyn 2D Interface for GiD}]
#         set GenText [= {Tdyn 2D Interface for GiD}]
#         set Developer [= {Tdyn 2D (C) COMPASS IS}]
#         set Company [ join {"COMPASS Ingeniera y Sistemas\n" "Barcelona\n" "(Spain)"} ""]
#         set Version [= {Ver. %s} $VersionNumber]
#         set Adress "C/ Manuel Girona 61, bajos\ne-mail: info@compassis.com\nfax: +34-93-2041909"
#     } else {
#         set ProgName [= {Tdyn 2DA Interface for GiD}]
#         set GenText [= {Tdyn 2DA Interface for GiD}]
#         set Developer [= {Tdyn 2DA (C) COMPASS IS}]
#         set Company [ join {"COMPASS Ingeniera y Sistemas\n" "Barcelona\n" "(Spain)"} ""]
#         set Version [= {Ver. %s} $VersionNumber]
#         set Adress "C/ Manuel Girona 61, bajos\ne-mail: info@compassis.com\nfax: +34-93-2041909"
#     }
# 
#     set im [image create photo -file [ file join $dir images/tdyn.gif]]
#     
#     toplevel .gid.pepe
#     wm title .gid.pepe $ProgName
# 
#     wm geometry .gid.pepe +[expr [winfo x $overw]+[winfo width $overw]/2-400]+[ \
#             expr [winfo y $overw]+[winfo height $overw]/2-250]
# 
#     frame .gid.pepe.mainf -bd 2 -relief ridge
# 
#     frame .gid.pepe.mainf1 -bd 1 -relief ridge
# 
#     message .gid.pepe.l -justify center -text $ProgName -width 200 -font BigFont
# 
#     label .gid.pepe.l2 -text $GenText -justify center -wraplength 200
#     label .gid.pepe.l23 -image $im -relief ridge -bd 2
#     
#     frame .gid.pepe.mainf2 -bd 1 -relief ridge
# 
#     label .gid.pepe.l25 -text $Developer -justify left
#     label .gid.pepe.l27 -text $Company -wraplength 200
#     label .gid.pepe.l30 -text $Adress -justify left
#     label .gid.pepe.l3 -text $Version
#     frame .gid.pepe.b
#     button .gid.pepe.b.ok -text [= {OK}] -command "destroy .gid.pepe" -padx 10
# #    button .gid.pepe.b.more -text [= {More}] -command "AddMoreWelcome"
#     pack .gid.pepe.b.ok -padx 3 -side left -anchor center -pady 3
# #    pack  .gid.pepe.l30 .gid.pepe.l27 .gid.pepe.l25 .gid.pepe.l23 .gid.pepe.l3 .gid.pepe.l2 \
# #            .gid.pepe.l -in .gid.pepe.mainf -side bottom -pady 5 -fill x -expand yes
# 
#     pack .gid.pepe.l30 .gid.pepe.l27 .gid.pepe.l25 -in .gid.pepe.mainf2 -side bottom -pady 5 -fill both -expand yes
#     pack .gid.pepe.l23 .gid.pepe.l3 .gid.pepe.l -in .gid.pepe.mainf1 -side bottom -pady 5 -fill both -expand yes
# 
#     pack .gid.pepe.mainf1 .gid.pepe.mainf2 -in .gid.pepe.mainf -side left -fill both -expand yes
# 
#     pack .gid.pepe.b -side bottom
#     pack .gid.pepe.mainf -side top -expand yes -fill both
# 
#     tkwait visibility .gid.pepe
#     grab  .gid.pepe 
#     tkwait window .gid.pepe
# }




################################################################################
# StartCalculate is called when selecting Calculate remote or Calculate
#
################################################################################

proc StartCalculate { local_or_remote flagQuit} {
    global GIDDEFAULT VersionNumber ProgramName ProblemTypePriv
   
    # Set calculation type
    set ProblemTypePriv(calc_remote) $local_or_remote

    # Determina el nombre del archivo
    set aa [.central.s info Project]
    set ProjectName [lindex $aa 1]
    if { [file extension $ProjectName] == ".gid" } {
    set ProjectName [file root $ProjectName]
    }
    set basename [file tail $ProjectName]

    if { [GiD_AccessValue get gendata Solve_FluidFlow_in_Fluid] == "No" && 
	 [GiD_AccessValue get gendata Solve_Temperature_in_Fluid] == "No" && 
	 [GiD_AccessValue get gendata Solve_SpeciesConcentration_in_Fluid] == "No" && 
	 [GiD_AccessValue get gendata Solve_PhiProblems_in_Fluid] == "No" &&
	 [GiD_AccessValue get gendata Solve_FluidFlow_in_Solid] == "No" && 
	 [GiD_AccessValue get gendata Solve_Temperature_in_Solid] == "No" && 
	 [GiD_AccessValue get gendata Solve_SpeciesConcentration_in_Solid] == "No" && 
	 [GiD_AccessValue get gendata Solve_PhiProblems_in_Solid] == "No" && 
	 [GiD_AccessValue get gendata Solve_StructuralProblem_in_Solid] == "No" } {
	tk_dialogRAM .gid.tempwin [= {Error window}]\
	    "[= {Select problem to be solved in window 'Data > Problem Data > PROBLEM'}]." \
	    error 0 OK
	return
    }

    #   Assembling type definition
    set Fluid_Assembling_Type [GiD_AccessValue get gendata Fluid_Assembling_Type]
    if { ($ProblemTypePriv(fluid_atype) ne "1") && ($ProblemTypePriv(fluid_atype) ne "2") \
	    && ($ProblemTypePriv(fluid_atype) ne "3") } {
	set ProblemTypePriv(fluid_atype) 1
    }
    set Solid_Assembling_Type [GiD_AccessValue get gendata Solid_Assembling_Type]
    if { ($ProblemTypePriv(solid_atype) ne "1") && ($ProblemTypePriv(solid_atype) ne "2") \
	    && ($ProblemTypePriv(solid_atype) ne "3") } {
	set ProblemTypePriv(solid_atype) 1
    }
    if { ($Fluid_Assembling_Type ne "1") && ($Fluid_Assembling_Type ne "2") \
	    && ($Fluid_Assembling_Type ne "3") } {
	set Fluid_Assembling_Type $ProblemTypePriv(fluid_atype)
	#GiD_AccessValue set gendata Fluid_Assembling_Type $Fluid_Assembling_Type
	EditProblemData Fluid_Assembling_Type $ProblemTypePriv(fluid_atype) 
    }
    if { ($Solid_Assembling_Type ne "1") && ($Solid_Assembling_Type ne "2") \
	    && ($Solid_Assembling_Type ne "3") } {
	set Solid_Assembling_Type $ProblemTypePriv(solid_atype)
	#GiD_AccessValue set gendata Solid_Assembling_Type $Solid_Assembling_Type
	EditProblemData Solid_Assembling_Type $ProblemTypePriv(solid_atype)
    }
    if { ($Fluid_Assembling_Type != $ProblemTypePriv(fluid_atype)) || \
	($Solid_Assembling_Type != $ProblemTypePriv(solid_atype))} {
	if { [GiveProjectName] != "UNNAMED" } {
	    set answer [tk_messageBox -message "[= {This model was created with different assembling algorithm}].\n\
		    [= {Check current value in 'Utilities>Tdyn Preferences...>Algorithm'}].\n\
		    [= {Do you want to proceed?}]" \
		    -title [= {Assembling Preferences}] \
		    -type okcancel -icon question]
	    switch $answer {
		ok {
		    EditProblemData Fluid_Assembling_Type $ProblemTypePriv(fluid_atype) 
		    EditProblemData Solid_Assembling_Type $ProblemTypePriv(solid_atype)
		}
		cancel return
	    }
	}
    }

    #   Check if results file exists
    if { [ReadGiDVariable UseMoreWindows] > 0 } {
	set directory $ProjectName.gid
	if { [file pathtype $directory] == "relative" } {
	    set directory [file join [pwd] $directory]
	}
	set file [file join $directory $basename.flavia.res]
	
	if { [file exists $file] } {
	    set answer [tk_messageBox -message "[= {Results file exists}].\n[= {Do you want to overwrite it?}]" \
		    -title [= {Results file exists}] -type okcancel -icon question]
	    
	    switch $answer {
		ok {
		    .central.s process escape escape escape escape  Utilities Calculate escape
		    if { $flagQuit == 1 } {
		        set quitting [tk_messageBox -message "[= {Confirm}].\n" \
		                -title [= {Closing}] -type okcancel -icon question]
		        switch $quitting {
		            ok {
		                GiD_Process quit
		            }
		            cancel return
		        }  
		    }  
		}
		cancel return
	    }
	} else {
	    .central.s process escape escape escape escape Utilities Calculate escape
	    if { $flagQuit == 1 } {
		set quitting [tk_messageBox -message "[= {Confirm}].\n" \
		        -title [= {Closing}] -type okcancel -icon question]
		switch $quitting {
		    ok {
		        GiD_Process quit
		    }
		    cancel return
		}  
	    }  
	}
    } else {
	.central.s process escape escape escape escape Utilities Calculate escape
	if { $flagQuit == 1 } {
	    set quitting [tk_messageBox -message "[= {Confirm}].\n" \
		    -title [= {Closing}] -type okcancel -icon question]
	    switch $quitting {
		ok {
		    GiD_Process quit
		}
		cancel return
	    }  
	}  
    }
}

################################################################################
# SelectGIDBatFile is called automagically by GiD and must return the
# .bat file name or nothing
################################################################################

proc SelectGIDBatFile { directory basename } {
    global ProblemTypePriv

    if { $::2Dor2DA == "2D" } {
	if { [info exists ProblemTypePriv(calc_remote)] && \
		$ProblemTypePriv(calc_remote) eq "remote" } {
	    switch $::tcl_platform(platform) {
		"windows" {
		    return [list tdyn2d.win.remote.bat $::2Dor2DA \
		            [GiD_AccessValue get gendata Results_File]]
		}
		"default" {
		    WarnWin [= "Sorry, but remote calculation only works in Windows OS"]
		    return tdyn2d.unix.bat
		}
	    }
	} else {
	    switch $::tcl_platform(platform) {
		"windows" { 
		    return tdyn2d.win.bat
		}
		"default" { 
		    return tdyn2d.unix.bat
		}
	    }
	}
    } else {
	if { [info exists ProblemTypePriv(calc_remote)] && \
		$ProblemTypePriv(calc_remote) eq "remote" } {
	    switch $::tcl_platform(platform) {
		"windows" {
		    return [list tdyn2da.win.remote.bat $::2Dor2DA \
		            [GiD_AccessValue get gendata Results_File]]
		}
		"default" {
		    WarnWin [= "Sorry, but remote calculation only works in Windows OS"]
		    return tdyn2da.unix.bat
		}
	    }
	} else {
	    switch $::tcl_platform(platform) {
		"windows" { 
		    return tdyn2da.win.bat
		}
		"default" { 
		    return tdyn2da.unix.bat
		}
	    }
	}
    }
    return ""
}

############################################################################
#   Customized control-1 contextual menu
############################################################################

proc StartTdynContextualMenu { menu whatuse type entity } {
    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
    
    .central.s disable graphics 1
    .central.s disable warnline 1

    #   Problemas que se resolvern (preferences)
    set show_ransol [IsVarOnTdynDefaults show_ransol 1]
    set show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    set show_advect [IsVarOnTdynDefaults show_advect 1]
    set show_ursolver [IsVarOnTdynDefaults show_ursolver 1]
    set show_alemesh [IsVarOnTdynDefaults show_alemesh 1]
    set show_naval [IsVarOnTdynDefaults show_naval 1]
    set show_ramsolid [IsVarOnTdynDefaults show_ramsolid 1]
    set show_fluid [IsVarOnTdynDefaults show_fluid 1]
    set show_solid [IsVarOnTdynDefaults show_solid 1]
    set show_oddls [IsVarOnTdynDefaults show_oddls 1]

    switch $type Points { set ov ovpnt } Lines { set ov ovline } Surfaces { set ov ovsurf } \
	Volumes { set ov ovvol } Nodes { set ov ovpnt } Elements { set ov [list ovline ovsurf ovvol] }

    switch $type Points - Lines - Surfaces - Volumes { set where geometry } Nodes - \
	Elements { set where mesh }

    # Options for Boundary Conditions
    set books ""
    foreach ovi $ov {
	foreach cnd [.central.s info Conditions $ovi] {
	    set book [.central.s info Conditions $cnd BOOK]
	    lappend conds($book) $cnd
	    set varcnt "show_[string tolower $book]"
	    if { ![info exists $varcnt] } {
		set $varcnt 0
	    }
	    if { [lsearch $books $book] == -1 && [set $varcnt] } {
		lappend books $book
	    }
	}
    }
    if { $whatuse != "POSTUSE" } {
	if { [llength $books] > 0 } {
	    set j 1
	    while { [winfo exists $menu.m$j] } { incr j }
	    set submenu [menu $menu.m$j]
	    $menu add cascade -label [= {Conditions}] -menu $submenu
	    foreach book $books {
		set text $book
		if { $text == "Default" } { set text "" }
		ContextualEntity::AddToMenu $submenu [= {Open %s...} $text] [list GidOpenConditions $book]
	    }
	}
	$menu add separator
    }
    # Options for Materials and Boundaries
    if { $whatuse != "POSTUSE" } {
	set j 1
	while { [winfo exists $menu.m$j] } { incr j }
	set submenu1 [menu $menu.m$j]
	$menu add cascade -label [= {Materials}] -menu $submenu1
	incr j
	set submenu2 [menu $menu.m$j]
	$menu add cascade -label [= {Boundaries}] -menu $submenu2
	if { $show_fluid } {
	    ContextualEntity::AddToMenu $submenu1 [= {Fluid}] [list materials::init MATERIALS_(FLUID)]
	    ContextualEntity::AddToMenu $submenu2 [= {Fluid}] [list materials::init BOUNDARIES_(FLUID)]
	} 
	if { $show_solid } {
	    ContextualEntity::AddToMenu $submenu1 [= {Solid}] [list materials::init MATERIALS_(SOLID)]
	    ContextualEntity::AddToMenu $submenu2 [= {Solid}] [list materials::init BOUNDARIES_(SOLID)]
	} 
	if { $show_fluid && $show_solid && ($show_heatrans || $show_advect || $show_ursolver) } {
	    ContextualEntity::AddToMenu $submenu2 [= {Contact}] [list materials::init CONTACTS]
	}
	$menu add separator
    }

    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }

    return
}

proc CreateWelcomeTDYN { { overw .gid } } {
    
    global GIDDEFAULT GidPriv ProblemTypePriv VersionNumber
    global tcl_platform

    if { $::2Dor2DA == "2DA" } {
	set ProgName "[= {Tdyn 2DA Interface for GiD}]"
    } else {
	set ProgName "[= {Tdyn 2D Interface for GiD}]"
    }
    if { [winfo exists .gid.pepe] } {
	return
    }
    
    toplevel .gid.pepe
    wm title .gid.pepe $ProgName
    set base .gid.pepe
    wm minsize $base 380 210
    wm maxsize $base 380 10000
    
    wm geometry .gid.pepe +[expr [winfo x $overw]+[winfo width $overw]/2-315]+[ \
	    expr [winfo y $overw]+[winfo height $overw]/2-315]
    
    set imatge_about [image create photo -file \
	    [file join $ProblemTypePriv(problemtypedir) images tdyn-open_small.gif]]  
    
    if { $GidPriv(TextOfVersion) != "" } {
	set GidVersion $GidPriv(TextOfVersion)
    } else {
	set GidVersion [GiD_Info GiDVersion]
    }
    
    
    label $base.label_img -image $imatge_about 
    
    button $base.button_ok \
	-text [= "Close"] -width 11 -padx 2  -command "destroy .gid.pepe"
    button $base.button_info \
	-text [= "Info"] -width 11 -padx 2  -command "AddMoreTDYNWelcome"
    button $base.button_credits \
	-text [= "Credits"] -width 11 -padx 2  -command "AddTDYNCredits"
    
    #SetWidgetsWidthFromText $base.button_ok $base.button_info $base.button_credits 
    
    # Geometry management
    
    grid $base.label_img -in $base        -row 1 -column 1  \
	-columnspan 3
    
    grid $base.button_ok -in $base        -row 2 -column 1  \
	-sticky ew -padx 25 -pady 8
    grid $base.button_info -in $base      -row 2 -column 2  \
	-sticky ew -padx 25 -pady 8
    grid $base.button_credits -in $base   -row 2 -column 3  \
	-sticky ew -padx 25 -pady 8
    
    
    # ponemos el numero de version encima de la imagen
    canvas $base.c -borderwidth 0 -highlightthickness 0 -width 372 -height 25 -bg lightblue
    
    if { $tcl_platform(platform) == "windows" } {        
	place $base.c -x 4 -y 146 -in $base
    } else {
	place $base.c -x 4 -y 146 -in $base
    }
    
    # frame $base.fv -bd 0
    # label $base.fv.label_version -text $GidVersion -font BigFont -fg #cccccc -bg black
    # grid $base.fv.label_version -sticky we
    
    #set rw [ winfo reqwidth $base.fv.label_version]
    #place $base.label_version -x [ expr [ winfo reqwidth $base.label_img] - $rw - 16] -y 420 -in $base
    
    
    # Resize behavior management
    
    grid rowconfigure $base 1 -weight 0 -minsize 30
    grid rowconfigure $base 2 -weight 0 -minsize 30
    grid rowconfigure $base 3 -weight 1 
    
    
    grid columnconfigure $base 1 -weight 1 -minsize 30
    grid columnconfigure $base 2 -weight 1 -minsize 30
    grid columnconfigure $base 3 -weight 1 -minsize 30
    
    bind $base <Return> "destroy .gid.pepe"
    bind $base <Escape> "destroy .gid.pepe"
    
    
    tkwait visibility .gid.pepe
    grab  .gid.pepe 
    
    
    # a mover las letras
    update idletasks
    if { ![ winfo exist $base.c ]} {
	break
    }
    set txt "Version $VersionNumber"
    
    set len_txt [ string length $txt]
    set dest_x [ expr [ $base.c cget -width] - 36]
    for { set is 0} { $is < $len_txt} { incr is} {
	frame $base.fv$is -bd 0
	label $base.fv$is.l -text [ string index $txt [ expr $len_txt - $is - 1]] \
	    -font "Helvetica 13 bold italic" -fg #aaaaaa -bg lightblue -bd 0 -highlightthickness 0
	grid $base.fv$is.l -sticky we -padx 0 -pady 0 
	
	set rw [ winfo reqwidth $base.fv$is.l]
	set dest_x [ expr $dest_x - $rw]
	set ini_x 0
	set c_id [ $base.c create window $ini_x 0 -anchor nw -window $base.fv$is]
	set dif_x [ expr $dest_x - $ini_x]
	update
	update idletasks
	for { set ix $ini_x} { $ix < $dest_x} { set ix [ expr $ix + 5]} {
	    # por si han cerrado la ventana antes de acabar con las letritas
	    if { ![ winfo exist $base.c ]} {
		break
	    }
	    $base.c coords $c_id $ix 0 
	    update
	    update idletasks
	}
	# por si han cerrado la ventana antes de acabar con las letritas
	if { ![ winfo exist $base.c ]} {
	    break
	}
	$base.c coords $c_id $dest_x 0 
	
    }
    
    
    # por si han cerrado la ventana antes de acabar con las letritas
    if { [ winfo exist .gid.pepe]} {
	tkwait window .gid.pepe
    }
}

proc AddTDYNCredits {} {
    global GIDDEFAULTTCL tcl_platform ProblemTypePriv
    set base .gid.pepe
    
    if { [winfo exists $base.text] } {
	$base.text delete @0,0 end
	.gid.pepe.button_info conf -text [= "Info"] -command "AddMoreTDYNWelcome"
    } else {
	wm geometry .gid.pepe 380x400
	text $base.text \
	    -height 7 -width 1 -yscroll "$base.scrollbar set"
	
	scrollbar $base.scrollbar \
	    -orient v -command "$base.text yview"
    }
    $base.text ins end "TDYN\n"
    #         $base.text ins end "\t(C) Compass IS (http://www.compassis.com)\n"
    #         $base.text ins end "\nTDYN is a technology supported software by:\n"
    #         $base.text ins end "\tGlass Service (http://www.gsl.cz)\n"
    $base.text ins end "\tCompass Ingeniera y Sistemas (http://www.compassis.com)\n"
    #         $base.text ins end "\nTdyn (FEM Heat transfer analysis platform)\n"
    $base.text ins end "\t(C) Compass Ingeniera y Sistemas\n"
    $base.text ins end "\nGiD (Pre/postprocessing system)\n"
    $base.text ins end "\t(C) International Center for Numerical Methods in Engineering\n"
    #         $base.text ins end "\t(C) Compass Ingeniera y Sistemas\n"
    
    grid $base.text -in $base        -row 3 -column 1  \
	-columnspan 3 \
	-sticky nesw
    grid $base.scrollbar -in $base        -row 3 -column 3  \
	-sticky nse
    
    grid rowconfigure $base 3 -weight 1 
    
    $base.button_credits conf -text [= "Less"] -command { 
	destroy .gid.pepe.text
	destroy .gid.pepe.scrollbar
	wm geometry .gid.pepe 380x210
	
	.gid.pepe.button_credits conf -text [= "Credits"] -command "AddTDYNCredits" } 
}

proc AddMoreTDYNWelcome {} {
    global GIDDEFAULTTCL tcl_platform ProblemTypePriv
    set base .gid.pepe
    
    if { [winfo exists $base.text] } {
	$base.text delete @0,0 end
	.gid.pepe.button_credits conf -text [= "Credits"] -command "AddTDYNCredits"
    } else {
	wm geometry .gid.pepe 380x650
	text $base.text \
	    -height 7 -width 1 -yscroll "$base.scrollbar set"
	
	scrollbar $base.scrollbar \
	    -orient v -command "$base.text yview"
    }
    $base.text ins end "GID internal version:   "
    $base.text ins end "[GiD_Info GiDVersion] \n"
    $base.text ins end "Tdyn internal version:   "
    $base.text ins end "[TUI_exectdyntool [list -no_timetable \
		-tool give_version]] \n"
    #         $base.text ins end "Pbsim internal version:   "
    #         catch {
	#             set ExecFile [file join $ProblemTypePriv(problemtypedir) "../scripts" pbsim.exe]
	#             eval exec [list $ExecFile] -version
	#         } retv
    #         regexp {pbsim\s+(\S+)} $retv --> retv
    #         $base.text ins end "$retv\n"
    
    #$base.text ins end "Last compilation time:\n  "
    
    #if { ![ catch { set fcomp [ open [ file native [ file join $GIDDEFAULTTCL Compiled]]]}]} {
	#    while { ![ eof $fcomp] } {
	    #        $base.text ins end [ gets $fcomp]
	    #        $base.text ins end \n
	    #    }
	#    close $fcomp
	#}
    
    $base.text ins end "\ntcl_platform: "
    foreach i "machine platform os osVersion" {
	$base.text ins end " '$tcl_platform($i)'"
    }
    $base.text ins end \n
    
    catch { $base.text ins end [GiD_Info internalstate] }
    
    if { ![ catch { set fcomp [ open [ file native [ file join $GIDDEFAULTTCL Compiled-more]]]}] } {
	while { ![ eof $fcomp] } {
	    $base.text ins end [ gets $fcomp]
	    $base.text ins end \n
	}
	close $fcomp
    }
    
    grid $base.text -in $base        -row 3 -column 1  \
	-columnspan 3 \
	-sticky nesw
    grid $base.scrollbar -in $base        -row 3 -column 3  \
	-sticky nse
    
    grid rowconfigure $base 3 -weight 1 
    
    $base.button_info conf -text [= "Less"] -command { 
	destroy .gid.pepe.text
	destroy .gid.pepe.scrollbar
	wm geometry .gid.pepe 380x210
	
	.gid.pepe.button_info conf -text [= "Info"] -command "AddMoreTDYNWelcome" } 
}

proc Tdyn::Help { what } {
    global TdynPriv
    global ProgramTag GIDDEFAULT
    
    lassign [list "" ""] open_page select_in_tree
    switch $what {
	reference {
	    set dbfile [file join $TdynPriv(problemtypedir) .. info_Tdyn Tdyn_User_Manual.wnl]
	    set open_page "Tdyn Reference" ; set select_in_tree ""
	}
	gid_reference_manual -  gid_customization - gid_what_s_new - gid_FAQ {
	    set dbfile [file join $::GIDDEFAULT manuals $what.wnl]
	}
    }
    set ptdir $TdynPriv(problemtypedir)
    
    gid_groups_conds::open_help_viewer -ptdir $ptdir -open_page $open_page \
	-select_in_tree $select_in_tree -key compass99 $dbfile
    
    return
}



